@@ -26,14 +26,16 @@
 #include <alsa/control_external.h>
 
 #include <spa/utils/result.h>
+#include <spa/utils/string.h>
 #include <spa/utils/json.h>
 #include <spa/param/props.h>
 #include <spa/param/audio/format-utils.h>
 
 #include <pipewire/pipewire.h>
-#include <extensions/metadata.h>
+#include <pipewire/extensions/metadata.h>
 
-#define NAME "alsa-plugin"
+PW_LOG_TOPIC_STATIC(alsa_log_topic, "alsa.ctl");
+#define PW_LOG_TOPIC_DEFAULT alsa_log_topic
 
 #define VOLUME_MAX 65536
 
@@ -97,7 +99,7 @@ struct global {
 
 	snd_ctl_pipewire_t *ctl;
 
-	struct global_info *ginfo;
+	const struct global_info *ginfo;
 
 	uint32_t id;
 	uint32_t permissions;
@@ -140,13 +142,17 @@ static void do_resync(snd_ctl_pipewire_t *ctl)
 
 static int wait_resync(snd_ctl_pipewire_t *ctl)
 {
+	int res;
 	do_resync(ctl);
 
 	while (true) {
 		pw_thread_loop_wait(ctl->mainloop);
 
-		if (ctl->error)
-			return ctl->error;
+		res = ctl->error;
+		if (res < 0) {
+			ctl->error = 0;
+			return res;
+		}
 
 		if (ctl->pending_seq == ctl->last_seq)
 			break;
@@ -163,11 +169,11 @@ static struct global *find_global(snd_ctl_pipewire_t *ctl, uint32_t id,
 
 	spa_list_for_each(g, &ctl->globals, link) {
 		if ((g->id == id || g->id == name_id) &&
-		    (type == NULL || strcmp(g->ginfo->type, type) == 0))
+		    (type == NULL || spa_streq(g->ginfo->type, type)))
 			return g;
 		if (name != NULL && name[0] != '\0' &&
 		    (str = pw_properties_get(g->props, PW_KEY_NODE_NAME)) != NULL &&
-		    strcmp(name, str) == 0)
+		    spa_streq(name, str))
 			return g;
 	}
 	return NULL;
@@ -177,7 +183,7 @@ static struct global *find_best_node(snd_ctl_pipewire_t *ctl, uint32_t flags)
 {
 	struct global *g, *best = NULL;
 	spa_list_for_each(g, &ctl->globals, link) {
-		if ((strcmp(g->ginfo->type, PW_TYPE_INTERFACE_Node) == 0) &&
+		if ((spa_streq(g->ginfo->type, PW_TYPE_INTERFACE_Node)) &&
 		    (flags == 0 || (g->node.flags & flags) == flags) &&
 		    (best == NULL || best->node.priority < g->node.priority))
 			best = g;
@@ -268,9 +274,9 @@ static int pipewire_elem_count(snd_ctl_ext_t * ext)
 
 	pw_thread_loop_lock(ctl->mainloop);
 
-
 	err = ctl->error;
 	if (err < 0) {
+		ctl->error = 0;
 		count = err;
 		goto finish;
 	}
@@ -307,8 +313,10 @@ static int pipewire_elem_list(snd_ctl_ext_t * ext, unsigned int offset,
 	pw_thread_loop_lock(ctl->mainloop);
 
 	err = ctl->error;
-	if (err < 0)
+	if (err < 0) {
+		ctl->error = 0;
 		goto finish;
+	}
 
 	if (ctl->default_source[0] != '\0') {
 		if (offset == 0)
@@ -344,13 +352,13 @@ static snd_ctl_ext_key_t pipewire_find_elem(snd_ctl_ext_t * ext,
 
 	name = snd_ctl_elem_id_get_name(id);
 
-	if (strcmp(name, SOURCE_VOL_NAME) == 0)
+	if (spa_streq(name, SOURCE_VOL_NAME))
 		return 0;
-	if (strcmp(name, SOURCE_MUTE_NAME) == 0)
+	if (spa_streq(name, SOURCE_MUTE_NAME))
 		return 1;
-	if (strcmp(name, SINK_VOL_NAME) == 0)
+	if (spa_streq(name, SINK_VOL_NAME))
 		return 2;
-	if (strcmp(name, SINK_MUTE_NAME) == 0)
+	if (spa_streq(name, SINK_MUTE_NAME))
 		return 3;
 
 	return SND_CTL_EXT_KEY_NOT_FOUND;
@@ -374,8 +382,10 @@ static int pipewire_get_attribute(snd_ctl_ext_t * ext, snd_ctl_ext_key_t key,
 	pw_thread_loop_lock(ctl->mainloop);
 
 	err = ctl->error;
-	if (err < 0)
+	if (err < 0) {
+		ctl->error = 0;
 		goto finish;
+	}
 
 	err = pipewire_update_volume(ctl);
 	if (err < 0)
@@ -428,8 +438,10 @@ static int pipewire_read_integer(snd_ctl_ext_t * ext, snd_ctl_ext_key_t key,
 	pw_thread_loop_lock(ctl->mainloop);
 
 	err = ctl->error;
-	if (err < 0)
+	if (err < 0) {
+		ctl->error = 0;
 		goto finish;
+	}
 
 	err = pipewire_update_volume(ctl);
 	if (err < 0)
@@ -491,7 +503,7 @@ static struct spa_pod *build_volume_mute(struct spa_pod_builder *b, struct volum
 
 static int set_volume_mute(snd_ctl_pipewire_t *ctl, const char *name, struct volume *volume, int *mute)
 {
-	struct global *g, *dg;
+	struct global *g, *dg = NULL;
 	uint32_t id = SPA_ID_INVALID, device_id = SPA_ID_INVALID;
 	char buf[1024];
 	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buf, sizeof(buf));
@@ -511,7 +523,7 @@ static int set_volume_mute(snd_ctl_pipewire_t *ctl, const char *name, struct vol
 		device_id = g->node.profile_device_id;
 	}
 	pw_log_debug("id %d device_id %d flags:%08x", id, device_id, g->node.flags);
-	if (id != SPA_ID_INVALID && device_id != SPA_ID_INVALID) {
+	if (id != SPA_ID_INVALID && device_id != SPA_ID_INVALID && dg != NULL) {
 		if (!SPA_FLAG_IS_SET(dg->permissions, PW_PERM_W | PW_PERM_X))
 			return -EPERM;
 
@@ -558,8 +570,10 @@ static int pipewire_write_integer(snd_ctl_ext_t * ext, snd_ctl_ext_key_t key,
 	pw_thread_loop_lock(ctl->mainloop);
 
 	err = ctl->error;
-	if (err < 0)
+	if (err < 0) {
+		ctl->error = 0;
 		goto finish;
+	}
 
 	err = pipewire_update_volume(ctl);
 	if (err < 0)
@@ -599,17 +613,19 @@ static int pipewire_write_integer(snd_ctl_ext_t * ext, snd_ctl_ext_key_t key,
 			vol->values[i] = value[i];
 
 		if (key == 0)
-			set_volume_mute(ctl, ctl->default_source, vol, NULL);
+			err = set_volume_mute(ctl, ctl->default_source, vol, NULL);
 		else
-			set_volume_mute(ctl, ctl->default_sink, vol, NULL);
+			err = set_volume_mute(ctl, ctl->default_sink, vol, NULL);
 	} else {
 		if (key == 1)
-			set_volume_mute(ctl, ctl->default_source, NULL, &ctl->source_muted);
+			err = set_volume_mute(ctl, ctl->default_source, NULL, &ctl->source_muted);
 		else
-			set_volume_mute(ctl, ctl->default_sink, NULL, &ctl->sink_muted);
+			err = set_volume_mute(ctl, ctl->default_sink, NULL, &ctl->sink_muted);
 	}
+	if (err < 0)
+		goto finish;
 
-	wait_resync(ctl);
+	err = wait_resync(ctl);
 
 	if (err < 0)
 		goto finish;
@@ -653,8 +669,10 @@ static int pipewire_read_event(snd_ctl_ext_t * ext, snd_ctl_elem_id_t * id,
 	pw_thread_loop_lock(ctl->mainloop);
 
 	err = ctl->error;
-	if (err < 0)
+	if (err < 0) {
+		ctl->error = 0;
 		goto finish;
+	}
 
 	if (!ctl->updated || !ctl->subscribed) {
 		err = -EAGAIN;
@@ -682,12 +700,12 @@ static int pipewire_read_event(snd_ctl_ext_t * ext, snd_ctl_elem_id_t * id,
 
 	*event_mask = SND_CTL_EVENT_MASK_VALUE;
 
-	if (!ctl->updated)
-		poll_deactivate(ctl);
-
 	err = 1;
 
 finish:
+	if (!ctl->updated)
+		poll_deactivate(ctl);
+
 	pw_thread_loop_unlock(ctl->mainloop);
 
 	return err;
@@ -708,8 +726,10 @@ static int pipewire_ctl_poll_revents(snd_ctl_ext_t * ext, struct pollfd *pfd,
 	pw_thread_loop_lock(ctl->mainloop);
 
 	err = ctl->error;
-	if (err < 0)
+	if (err < 0) {
+		ctl->error = 0;
 		goto finish;
+	}
 
 	if (ctl->updated)
 		*revents = POLLIN;
@@ -729,7 +749,7 @@ static void snd_ctl_pipewire_free(snd_ctl_pipewire_t *ctl)
 	if (ctl == NULL)
 		return;
 
-	pw_log_debug(NAME" %p:", ctl);
+	pw_log_debug("%p:", ctl);
 	if (ctl->mainloop)
 		pw_thread_loop_stop(ctl->mainloop);
 	if (ctl->registry)
@@ -799,12 +819,14 @@ static void parse_props(struct global *g, const struct spa_pod *param, bool devi
 	SPA_POD_OBJECT_FOREACH(obj, prop) {
 		switch (prop->key) {
 		case SPA_PROP_volume:
-			spa_pod_get_float(&prop->value, &g->node.volume);
+			if (spa_pod_get_float(&prop->value, &g->node.volume) < 0)
+				continue;
 			pw_log_debug("update node %d volume", g->id);
 			SPA_FLAG_UPDATE(g->node.flags, NODE_FLAG_DEVICE_VOLUME, device);
 			break;
 		case SPA_PROP_mute:
-			spa_pod_get_bool(&prop->value, &g->node.mute);
+			if (spa_pod_get_bool(&prop->value, &g->node.mute) < 0)
+				continue;
 			SPA_FLAG_UPDATE(g->node.flags, NODE_FLAG_DEVICE_MUTE, device);
 			pw_log_debug("update node %d mute", g->id);
 			break;
@@ -834,7 +856,7 @@ static struct global *find_node_for_route(snd_ctl_pipewire_t *ctl, uint32_t card
 {
 	struct global *n;
 	spa_list_for_each(n, &ctl->globals, link) {
-		if (strcmp(n->ginfo->type, PW_TYPE_INTERFACE_Node) == 0 &&
+		if (spa_streq(n->ginfo->type, PW_TYPE_INTERFACE_Node) &&
 		    (n->node.device_id == card) &&
 		    (n->node.profile_device_id == device))
 			return n;
@@ -854,14 +876,14 @@ static void device_event_param(void *object, int seq,
 	switch (id) {
 	case SPA_PARAM_Route:
 	{
-		uint32_t index, device;
+		uint32_t idx, device;
 		enum spa_direction direction;
 		struct spa_pod *props = NULL;
 		struct global *ng;
 
 		if (spa_pod_parse_object(param,
 				SPA_TYPE_OBJECT_ParamRoute, NULL,
-				SPA_PARAM_ROUTE_index, SPA_POD_Int(&index),
+				SPA_PARAM_ROUTE_index, SPA_POD_Int(&idx),
 				SPA_PARAM_ROUTE_direction, SPA_POD_Id(&direction),
 				SPA_PARAM_ROUTE_device, SPA_POD_Int(&device),
 				SPA_PARAM_ROUTE_props, SPA_POD_OPT_Pod(&props)) < 0) {
@@ -869,13 +891,13 @@ static void device_event_param(void *object, int seq,
 			return;
 		}
 		if (direction == SPA_DIRECTION_OUTPUT)
-			g->device.active_route_output = index;
+			g->device.active_route_output = idx;
                 else
-                        g->device.active_route_input = index;
+                        g->device.active_route_input = idx;
 
 		pw_log_debug("device %d: active %s route %d", g->id,
 				direction == SPA_DIRECTION_OUTPUT ? "output" : "input",
-				index);
+				idx);
 
 		ng = find_node_for_route(ctl, g->id, device);
 		if (props && ng)
@@ -893,7 +915,7 @@ static const struct pw_device_events device_events = {
 	.param = device_event_param,
 };
 
-struct global_info device_info = {
+static const struct global_info device_info = {
 	.type = PW_TYPE_INTERFACE_Device,
 	.version = PW_VERSION_DEVICE,
 	.events = &device_events,
@@ -920,12 +942,12 @@ static void node_event_info(void *object, const struct pw_node_info *info)
 		else
 			g->node.device_id = SPA_ID_INVALID;
 
-		if ((str = spa_dict_lookup(info->props, PW_KEY_PRIORITY_DRIVER)))
+		if ((str = spa_dict_lookup(info->props, PW_KEY_PRIORITY_SESSION)))
 			g->node.priority = atoi(str);
 		if ((str = spa_dict_lookup(info->props, PW_KEY_MEDIA_CLASS))) {
-			if (strcmp(str, "Audio/Sink") == 0)
+			if (spa_streq(str, "Audio/Sink"))
 				g->node.flags |= NODE_FLAG_SINK;
-			else if (strcmp(str, "Audio/Source") == 0)
+			else if (spa_streq(str, "Audio/Source"))
 				g->node.flags |= NODE_FLAG_SOURCE;
 		}
 	}
@@ -971,7 +993,7 @@ static const struct pw_node_events node_events = {
 	.param = node_event_param,
 };
 
-struct global_info node_info = {
+static const struct global_info node_info = {
 	.type = PW_TYPE_INTERFACE_Node,
 	.version = PW_VERSION_NODE,
 	.events = &node_events,
@@ -989,7 +1011,7 @@ static int json_object_find(const char *obj, const char *key, char *value, size_
 		return -EINVAL;
 
 	while (spa_json_get_string(&it[1], k, sizeof(k)-1) > 0) {
-		if (strcmp(k, key) == 0) {
+		if (spa_streq(k, key)) {
 			if (spa_json_get_string(&it[1], value, len) <= 0)
 				continue;
 			return 0;
@@ -1011,14 +1033,14 @@ static int metadata_property(void *object,
 	snd_ctl_pipewire_t *ctl = g->ctl;
 
 	if (subject == PW_ID_CORE) {
-		if (key == NULL || strcmp(key, "default.audio.sink") == 0) {
+		if (key == NULL || spa_streq(key, "default.audio.sink")) {
 			if (value == NULL ||
 			    json_object_find(value, "name",
 					ctl->default_sink, sizeof(ctl->default_sink)) < 0)
 				ctl->default_sink[0] = '\0';
 			pw_log_debug("found default sink: %s", ctl->default_sink);
 		}
-		if (key == NULL || strcmp(key, "default.audio.source") == 0) {
+		if (key == NULL || spa_streq(key, "default.audio.source")) {
 			if (value == NULL ||
 			    json_object_find(value, "name",
 					ctl->default_source, sizeof(ctl->default_source)) < 0)
@@ -1041,7 +1063,7 @@ static const struct pw_metadata_events metadata_events = {
 	.property = metadata_property,
 };
 
-struct global_info metadata_info = {
+static const struct global_info metadata_info = {
 	.type = PW_TYPE_INTERFACE_Metadata,
 	.version = PW_VERSION_METADATA,
 	.events = &metadata_events,
@@ -1073,30 +1095,34 @@ static void registry_event_global(void *data, uint32_t id,
 		const struct spa_dict *props)
 {
 	snd_ctl_pipewire_t *ctl = data;
-	struct global_info *info = NULL;
+	const struct global_info *info = NULL;
 	struct pw_proxy *proxy;
 	const char *str;
 
 	pw_log_debug("got %d %s", id, type);
 
-	if (strcmp(type, PW_TYPE_INTERFACE_Device) == 0) {
+	if (spa_streq(type, PW_TYPE_INTERFACE_Device)) {
 		if (props == NULL ||
 		    ((str = spa_dict_lookup(props, PW_KEY_MEDIA_CLASS)) == NULL) ||
-		    (strcmp(str, "Audio/Device") != 0))
+		    (!spa_streq(str, "Audio/Device")))
 			return;
 
 		pw_log_debug("found device %d", id);
 		info = &device_info;
-	} else if (strcmp(type, PW_TYPE_INTERFACE_Node) == 0) {
+	} else if (spa_streq(type, PW_TYPE_INTERFACE_Node)) {
 		if (props == NULL ||
 		    ((str = spa_dict_lookup(props, PW_KEY_MEDIA_CLASS)) == NULL) ||
-		    ((strcmp(str, "Audio/Sink") != 0) &&
-		     (strcmp(str, "Audio/Source") != 0)))
+		    ((!spa_streq(str, "Audio/Sink")) &&
+		     (!spa_streq(str, "Audio/Source"))))
 			return;
 
 		pw_log_debug("found node %d type:%s", id, str);
 		info = &node_info;
-	} else if (strcmp(type, PW_TYPE_INTERFACE_Metadata) == 0) {
+	} else if (spa_streq(type, PW_TYPE_INTERFACE_Metadata)) {
+		if (props == NULL ||
+		    ((str = spa_dict_lookup(props, PW_KEY_METADATA_NAME)) == NULL) ||
+		    (!spa_streq(str, "default")))
+			return;
 		if (ctl->metadata != NULL)
 			return;
 		info = &metadata_info;
@@ -1144,9 +1170,9 @@ static void registry_event_global_remove(void *data, uint32_t id)
 	if ((name = pw_properties_get(g->props, PW_KEY_NODE_NAME)) == NULL)
 		return;
 
-	if (strcmp(name, ctl->default_sink) == 0)
+	if (spa_streq(name, ctl->default_sink))
 		ctl->default_sink[0] = '\0';
-	if (strcmp(name, ctl->default_source) == 0)
+	if (spa_streq(name, ctl->default_source))
 		ctl->default_source[0] = '\0';
 }
 
@@ -1160,13 +1186,18 @@ static void on_core_error(void *data, uint32_t id, int seq, int res, const char
 {
 	snd_ctl_pipewire_t *ctl = data;
 
-	pw_log_warn(NAME" %p: error id:%u seq:%d res:%d (%s): %s", ctl,
+	pw_log_warn("%p: error id:%u seq:%d res:%d (%s): %s", ctl,
 			id, seq, res, spa_strerror(res), message);
 
 	if (id == PW_ID_CORE) {
-		ctl->error = res;
-		if (ctl->fd != -1)
-			poll_activate(ctl);
+		switch (res) {
+		case -ENOENT:
+			break;
+		default:
+			ctl->error = res;
+			if (ctl->fd != -1)
+				poll_activate(ctl);
+		}
 	}
 	pw_thread_loop_signal(ctl->mainloop, false);
 }
@@ -1180,7 +1211,7 @@ static void on_core_done(void *data, uint32_t id, int seq)
 	if (id != PW_ID_CORE)
 		return;
 
-	ctl->last_seq = ctl->pending_seq;
+	ctl->last_seq = seq;
 	if (seq == ctl->pending_seq) {
 		pipewire_update_volume(ctl);
 		pw_thread_loop_signal(ctl->mainloop, false);
@@ -1211,15 +1242,17 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 
         pw_init(NULL, NULL);
 
+	PW_LOG_TOPIC_INIT(alsa_log_topic);
+
 	snd_config_for_each(i, next, conf) {
 		snd_config_t *n = snd_config_iterator_entry(i);
 		const char *id;
 		if (snd_config_get_id(n, &id) < 0)
 			continue;
-		if (strcmp(id, "comment") == 0 || strcmp(id, "type") == 0
-		    || strcmp(id, "hint") == 0)
+		if (spa_streq(id, "comment") || spa_streq(id, "type")
+		    || spa_streq(id, "hint"))
 			continue;
-		if (strcmp(id, "server") == 0) {
+		if (spa_streq(id, "server")) {
 			if (snd_config_get_string(n, &server) < 0) {
 				SNDERR("Invalid type for %s", id);
 				return -EINVAL;
@@ -1228,7 +1261,7 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 			}
 			continue;
 		}
-		if (strcmp(id, "device") == 0) {
+		if (spa_streq(id, "device")) {
 			if (snd_config_get_string(n, &device) < 0) {
 				SNDERR("Invalid type for %s", id);
 				return -EINVAL;
@@ -1237,7 +1270,7 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 			}
 			continue;
 		}
-		if (strcmp(id, "source") == 0) {
+		if (spa_streq(id, "source")) {
 			if (snd_config_get_string(n, &source) < 0) {
 				SNDERR("Invalid type for %s", id);
 				return -EINVAL;
@@ -1246,7 +1279,7 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 			}
 			continue;
 		}
-		if (strcmp(id, "sink") == 0) {
+		if (spa_streq(id, "sink")) {
 			if (snd_config_get_string(n, &sink) < 0) {
 				SNDERR("Invalid type for %s", id);
 				return -EINVAL;
@@ -1255,7 +1288,7 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 			}
 			continue;
 		}
-		if (strcmp(id, "fallback") == 0) {
+		if (spa_streq(id, "fallback")) {
 			if (snd_config_get_string(n, &fallback_name) < 0) {
 				SNDERR("Invalid value for %s", id);
 				return -EINVAL;
@@ -1270,7 +1303,7 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 	if (str != NULL && str[0] != '\0')
 		server = str;
 
-	if (fallback_name && name && !strcmp(name, fallback_name))
+	if (fallback_name && name && spa_streq(name, fallback_name))
 		fallback_name = NULL; /* no fallback for the same name */
 
 	ctl = calloc(1, sizeof(*ctl));
@@ -1346,7 +1379,9 @@ SND_CTL_PLUGIN_DEFINE_FUNC(pipewire)
 			&ctl->registry_listener,
 			&registry_events, ctl);
 
-	wait_resync(ctl);
+	err = wait_resync(ctl);
+	if (err < 0)
+		goto error_unlock;
 
 	pw_thread_loop_unlock(ctl->mainloop);
 
