@@ -314,10 +314,12 @@ static inline const char *spa_bt_profile_name (enum spa_bt_profile profile) {
 
 struct spa_bt_monitor;
 struct spa_bt_backend;
+struct spa_bt_player;
 
 struct spa_bt_adapter {
 	struct spa_list link;
 	struct spa_bt_monitor *monitor;
+	struct spa_bt_player *dummy_player;
 	char *path;
 	char *alias;
 	char *address;
@@ -332,6 +334,7 @@ struct spa_bt_adapter {
 	int powered;
 	unsigned int endpoints_registered:1;
 	unsigned int application_registered:1;
+	unsigned int player_registered:1;
 	unsigned int has_battery_provider;
 	unsigned int battery_provider_unavailable;
 };
@@ -678,6 +681,7 @@ enum spa_bt_feature {
 	SPA_BT_FEATURE_HW_VOLUME	= (1 << 3),
 	SPA_BT_FEATURE_HW_VOLUME_MIC	= (1 << 4),
 	SPA_BT_FEATURE_SBC_XQ		= (1 << 5),
+	SPA_BT_FEATURE_FASTSTREAM	= (1 << 6),
 };
 
 struct spa_bt_quirks;
@@ -696,14 +700,15 @@ struct spa_bt_backend_implementation {
 	int (*free) (void *data);
 	int (*register_profiles) (void *data);
 	int (*unregister_profiles) (void *data);
-	int (*unregistered) (void *data);
-	int (*add_filters) (void *data);
 	int (*ensure_codec) (void *data, struct spa_bt_device *device, unsigned int codec);
 	int (*supports_codec) (void *data, struct spa_bt_device *device, unsigned int codec);
 };
 
 struct spa_bt_backend {
 	struct spa_callbacks impl;
+	const char *name;
+	bool available;
+	bool exclusive;
 };
 
 #define spa_bt_backend_set_implementation(b,_impl,_data) \
@@ -722,8 +727,6 @@ struct spa_bt_backend {
 #define spa_bt_backend_free(b)			spa_bt_backend_impl(b, free, 0)
 #define spa_bt_backend_register_profiles(b)	spa_bt_backend_impl(b, register_profiles, 0)
 #define spa_bt_backend_unregister_profiles(b)	spa_bt_backend_impl(b, unregister_profiles, 0)
-#define spa_bt_backend_unregistered(b)		spa_bt_backend_impl(b, unregistered, 0)
-#define spa_bt_backend_add_filters(b)		spa_bt_backend_impl(b, add_filters, 0)
 #define spa_bt_backend_ensure_codec(b,...)	spa_bt_backend_impl(b, ensure_codec, 0, __VA_ARGS__)
 #define spa_bt_backend_supports_codec(b,...)	spa_bt_backend_impl(b, supports_codec, 0, __VA_ARGS__)
 
