@@ -36,6 +36,7 @@
 #include <spa/utils/list.h>
 #include <spa/utils/keys.h>
 #include <spa/utils/names.h>
+#include <spa/utils/string.h>
 #include <spa/monitor/device.h>
 #include <spa/node/node.h>
 #include <spa/node/io.h>
@@ -48,8 +49,6 @@
 
 #include "libcamera.h"
 
-#define NAME "libcamera-source"
-
 static const char default_device[] = "/dev/media0";
 
 struct props {
@@ -337,23 +336,25 @@ static const struct spa_dict_item info_items[] = {
 
 static void emit_node_info(struct impl *this, bool full)
 {
+	uint64_t old = full ? this->info.change_mask : 0;
 	if (full)
 		this->info.change_mask = this->info_all;
 	if (this->info.change_mask) {
 		this->info.props = &SPA_DICT_INIT_ARRAY(info_items);
 		spa_node_emit_info(&this->hooks, &this->info);
-		this->info.change_mask = 0;
+		this->info.change_mask = old;
 	}
 }
 
 static void emit_port_info(struct impl *this, struct port *port, bool full)
 {
+	uint64_t old = full ? port->info.change_mask : 0;
 	if (full)
 		port->info.change_mask = port->info_all;
 	if (port->info.change_mask) {
 		spa_node_emit_port_info(&this->hooks,
 				SPA_DIRECTION_OUTPUT, 0, &port->info);
-		port->info.change_mask = 0;
+		port->info.change_mask = old;
 	}
 }
 
@@ -824,7 +825,7 @@ static int impl_node_process(void *object)
 	if (port->control)
 		process_control(this, &port->control->sequence);
 
-	spa_log_trace(this->log, NAME " %p; status %d", this, io->status);
+	spa_log_trace(this->log, "%p; status %d", this, io->status);
 
 	if (io->status == SPA_STATUS_HAVE_DATA) {
 		return SPA_STATUS_HAVE_DATA;
@@ -846,7 +847,7 @@ static int impl_node_process(void *object)
 	spa_list_remove(&b->link);
 	SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUTSTANDING);
 
-	spa_log_trace(this->log, NAME " %p: dequeue buffer %d", this, b->id);
+	spa_log_trace(this->log, "%p: dequeue buffer %d", this, b->id);
 
 	io->buffer_id = b->id;
 	io->status = SPA_STATUS_HAVE_DATA;
@@ -882,7 +883,7 @@ static int impl_get_interface(struct spa_handle *handle, const char *type, void
 
 	this = (struct impl *) handle;
 
-	if (strcmp(type, SPA_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(type, SPA_TYPE_INTERFACE_Node))
 		*interface = &this->node;
 	else
 		return -ENOENT;
@@ -937,6 +938,8 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
+	libcamera_log_topic_init(this->log);
+
 	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
 	this->system = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_System);
 
@@ -991,10 +994,10 @@ impl_init(const struct spa_handle_factory *factory,
 	port->dev.log = this->log;
 	port->dev.fd = -1;
 
-	if(port->dev.camera == NULL) {
+	if(port->dev.camera == NULL)
 		port->dev.camera = (LibCamera*)newLibCamera();
+	if(port->dev.camera != NULL)
 		libcamera_set_log(port->dev.camera, port->dev.log);
-	}
 
 	if (info && (str = spa_dict_lookup(info, SPA_KEY_API_LIBCAMERA_PATH))) {
 		strncpy(this->props.device, str, 63);
