@@ -486,11 +486,6 @@ extern "C" {
 	}
 
 	int LibCamera::start() {
-		if(!this->set_config()) {
-			return -1;
-		}
-
-		this->streamName_.clear();
 		for (unsigned int index = 0; index < this->config_->size(); ++index) {
 			StreamConfiguration &cfg = this->config_->at(index);
 			this->streamName_[cfg.stream()] = "stream" + std::to_string(index);
@@ -522,8 +517,6 @@ extern "C" {
 	}
 
 	void LibCamera::stop() {
-		this->disconnect();
-
 		StreamConfiguration &cfg = this->config_->at(0);
 		Stream *stream = cfg.stream();
 		uint32_t nbuffers = this->allocator_->buffers(stream).size();
@@ -536,16 +529,19 @@ extern "C" {
     	spa_log_info(this->log_, "Stopping camera ...");
     	this->cam_->stop();
     	if(this->allocator_) {
+		this->allocator_->free(stream);
 	    	delete this->allocator_;
 	    	this->allocator_ = nullptr;
     	}
 
 	    this->item_free_fn();
+	    this->requests_.clear();
+	    this->streamName_.clear();
 	}
 
 	void LibCamera::close() {
-    	this->stop();
-		this->cam_->release();
+		if (this->cam_)
+			this->cam_->release();
 	}
 
 	void LibCamera::connect()
@@ -582,6 +578,10 @@ extern "C" {
 		camera->set_streamcfgpixel_format(fmt);
 	}
 
+	bool libcamera_set_config(LibCamera *camera) {
+		return camera->set_config();
+	}
+
 	void libcamera_ringbuffer_read_update(LibCamera *camera) {
 		camera->ring_buffer_update_read_index();
 	}
@@ -776,6 +776,9 @@ extern "C" {
 		std::unique_ptr<CameraManager> cm = std::make_unique<CameraManager>();
 		LibCamera* camera = new LibCamera();
 
+		pthread_mutexattr_init(&attr);
+		pthread_mutex_init(&camera->lock, &attr);
+
 		ret = cm->start();
 		if (ret) {
 			deleteLibCamera(camera);
@@ -795,9 +798,6 @@ extern "C" {
 			return nullptr;
 		}
 
-		pthread_mutexattr_init(&attr);
-		pthread_mutex_init(&camera->lock, &attr);
-
 		camera->ring_buffer_init();
 
 		return camera;
