@@ -49,8 +49,6 @@
 
 #include "v4l2.h"
 
-#define NAME "v4l2-source"
-
 static const char default_device[] = "/dev/video0";
 
 struct props {
@@ -877,7 +875,7 @@ static int impl_node_process(void *object)
 	if (port->control)
 		process_control(this, &port->control->sequence);
 
-	spa_log_trace(this->log, NAME " %p; status %d", this, io->status);
+	spa_log_trace(this->log, "%p; status %d", this, io->status);
 
 	if (io->status == SPA_STATUS_HAVE_DATA)
 		return SPA_STATUS_HAVE_DATA;
@@ -896,7 +894,7 @@ static int impl_node_process(void *object)
 	spa_list_remove(&b->link);
 	SPA_FLAG_SET(b->flags, BUFFER_FLAG_OUTSTANDING);
 
-	spa_log_trace(this->log, NAME " %p: dequeue buffer %d", this, b->id);
+	spa_log_trace(this->log, "%p: dequeue buffer %d", this, b->id);
 
 	io->buffer_id = b->id;
 	io->status = SPA_STATUS_HAVE_DATA;
@@ -973,8 +971,9 @@ impl_init(const struct spa_handle_factory *factory,
 	this = (struct impl *) handle;
 
 	this->log = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_Log);
-	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
+	v4l2_log_topic_init(this->log);
 
+	this->data_loop = spa_support_find(support, n_support, SPA_TYPE_INTERFACE_DataLoop);
 	if (this->data_loop == NULL) {
 		spa_log_error(this->log, "a data_loop is needed");
 		return -EINVAL;
