@@ -39,10 +39,17 @@
 
 #include <pipewire/private.h>
 #include <pipewire/impl.h>
-#include <extensions/profiler.h>
+#include <pipewire/extensions/profiler.h>
+
+/** \page page_module_profiler PipeWire Module: Profiler
+ */
 
 #define NAME "profiler"
 
+PW_LOG_TOPIC(mod_topic, "mod." NAME);
+#define PW_LOG_TOPIC_DEFAULT mod_topic
+
+#define TMP_BUFFER		(16 * 1024)
 #define MAX_BUFFER		(8 * 1024 * 1024)
 #define MIN_FLUSH		(16 * 1024)
 #define DEFAULT_IDLE		5
@@ -79,7 +86,10 @@ struct impl {
 	unsigned int listening:1;
 
 	struct spa_ringbuffer buffer;
+	uint8_t tmp[TMP_BUFFER];
 	uint8_t data[MAX_BUFFER];
+
+	uint8_t flush[MAX_BUFFER + sizeof(struct spa_pod_struct)];
 };
 
 struct resource_data {
@@ -121,14 +131,14 @@ static void stop_flush(struct impl *impl)
 static void flush_timeout(void *data, uint64_t expirations)
 {
 	struct impl *impl = data;
-	int32_t avail, size;
+	int32_t avail;
 	uint32_t idx;
 	struct spa_pod_struct *p;
 	struct pw_resource *resource;
 
 	avail = spa_ringbuffer_get_read_index(&impl->buffer, &idx);
 
-	pw_log_trace(NAME"%p avail %d", impl, avail);
+	pw_log_trace("%p avail %d", impl, avail);
 
 	if (avail <= 0) {
 		if (++impl->empty == DEFAULT_IDLE)
@@ -137,13 +147,12 @@ static void flush_timeout(void *data, uint64_t expirations)
 	}
 	impl->empty = 0;
 
-	size = avail + sizeof(struct spa_pod_struct);
-	p = alloca(size);
+	p = (struct spa_pod_struct *)impl->flush;
 	*p = SPA_POD_INIT_Struct(avail);
 
 	spa_ringbuffer_read_data(&impl->buffer, impl->data, MAX_BUFFER,
 			idx % MAX_BUFFER,
-			SPA_MEMBER(p, sizeof(struct spa_pod_struct), void), avail);
+			SPA_PTROFF(p, sizeof(struct spa_pod_struct), void), avail);
 	spa_ringbuffer_read_update(&impl->buffer, idx + avail);
 
 	spa_list_for_each(resource, &impl->global->resource_list, link)
@@ -153,7 +162,6 @@ static void flush_timeout(void *data, uint64_t expirations)
 static void context_do_profile(void *data, struct pw_impl_node *node)
 {
 	struct impl *impl = data;
-	char buffer[4096];
 	struct spa_pod_builder b;
 	struct spa_pod_frame f[2];
 	struct pw_node_activation *a = node->rt.activation;
@@ -162,7 +170,10 @@ static void context_do_profile(void *data, struct pw_impl_node *node)
 	int32_t filled;
 	uint32_t idx, avail;
 
-	spa_pod_builder_init(&b, buffer, sizeof(buffer));
+	if (SPA_FLAG_IS_SET(pos->clock.flags, SPA_IO_CLOCK_FLAG_FREEWHEEL))
+		return;
+
+	spa_pod_builder_init(&b, impl->tmp, sizeof(impl->tmp));
 	spa_pod_builder_push_object(&b, &f[0],
 			SPA_TYPE_OBJECT_Profiler, 0);
 
@@ -187,6 +198,7 @@ static void context_do_profile(void *data, struct pw_impl_node *node)
 			SPA_POD_Double(pos->clock.rate_diff),
 			SPA_POD_Long(pos->clock.next_nsec));
 
+
 	spa_pod_builder_prop(&b, SPA_PROFILER_driverBlock, 0);
 	spa_pod_builder_add_struct(&b,
 			SPA_POD_Int(node->info.id),
@@ -219,14 +231,17 @@ static void context_do_profile(void *data, struct pw_impl_node *node)
 	}
 	spa_pod_builder_pop(&b, &f[0]);
 
+	if (b.state.offset > sizeof(impl->tmp))
+		goto done;
+
 	filled = spa_ringbuffer_get_write_index(&impl->buffer, &idx);
 	if (filled < 0 || filled > MAX_BUFFER) {
-		pw_log_warn(NAME " %p: queue xrun %d", impl, filled);
+		pw_log_warn("%p: queue xrun %d", impl, filled);
 		goto done;
 	}
 	avail = MAX_BUFFER - filled;
 	if (avail < b.state.offset) {
-		pw_log_warn(NAME " %p: queue full %d < %d", impl, avail, b.state.offset);
+		pw_log_warn("%p: queue full %d < %d", impl, avail, b.state.offset);
 		goto done;
 	}
 	spa_ringbuffer_write_data(&impl->buffer,
@@ -268,7 +283,7 @@ static void resource_destroy(void *data)
 {
 	struct impl *impl = data;
 	if (--impl->busy == 0) {
-		pw_log_info(NAME" %p: stopping profiler", impl);
+		pw_log_info("%p: stopping profiler", impl);
 		stop_listener(impl);
 	}
 }
@@ -311,7 +326,7 @@ global_bind(void *_data, struct pw_impl_client *client, uint32_t permissions,
 			&resource_events, impl);
 
 	if (++impl->busy == 1) {
-		pw_log_info(NAME" %p: starting profiler", impl);
+		pw_log_info("%p: starting profiler", impl);
 		pw_loop_invoke(impl->context->data_loop,
                        do_start, SPA_ID_INVALID, NULL, 0, false, impl);
 		impl->listening = true;
@@ -327,8 +342,7 @@ static void module_destroy(void *data)
 
 	spa_hook_remove(&impl->module_listener);
 
-	if (impl->properties)
-		pw_properties_free(impl->properties);
+	pw_properties_free(impl->properties);
 
 	free(impl);
 }
@@ -346,6 +360,8 @@ int pipewire__module_init(struct pw_impl_module *module, const char *args)
 	struct impl *impl;
 	struct pw_loop *main_loop = pw_context_get_main_loop(context);
 
+	PW_LOG_TOPIC_INIT(mod_topic);
+
 	impl = calloc(1, sizeof(struct impl));
 	if (impl == NULL)
 		return -errno;
