@@ -36,9 +36,11 @@
 
 #include "buffers.h"
 
-#define NAME "buffers"
+PW_LOG_TOPIC_EXTERN(log_buffers);
+#define PW_LOG_TOPIC_DEFAULT log_buffers
 
 #define MAX_ALIGN	32
+#define MAX_BLOCKS	64u
 
 struct port {
 	struct spa_node *node;
@@ -87,7 +89,7 @@ static int alloc_buffers(struct pw_mempool *pool,
 				SPA_PARAM_META_size, SPA_POD_Int(&size)) < 0)
 				continue;
 
-			pw_log_debug(NAME" %p: enable meta %d %d", allocation, type, size);
+			pw_log_debug("%p: enable meta %d %d", allocation, type, size);
 
 			metas[n_metas].type = type;
 			metas[n_metas].size = size;
@@ -120,7 +122,7 @@ static int alloc_buffers(struct pw_mempool *pool,
 	if (buffers == NULL)
 		return -errno;
 
-	skel = SPA_MEMBER(buffers, n_buffers * sizeof(struct spa_buffer *), void);
+	skel = SPA_PTROFF(buffers, n_buffers * sizeof(struct spa_buffer *), void);
 	skel = SPA_PTR_ALIGN(skel, info.max_align, void);
 
 	if (SPA_FLAG_IS_SET(flags, PW_BUFFERS_FLAG_SHARED)) {
@@ -131,8 +133,10 @@ static int alloc_buffers(struct pw_mempool *pool,
 				PW_MEMBLOCK_FLAG_MAP,
 				SPA_DATA_MemFd,
 				n_buffers * info.mem_size);
-		if (m == NULL)
+		if (m == NULL) {
+			free(buffers);
 			return -errno;
+		}
 
 		data = m->map->ptr;
 	} else {
@@ -140,7 +144,8 @@ static int alloc_buffers(struct pw_mempool *pool,
 		data = NULL;
 	}
 
-	pw_log_debug(NAME" %p: layout buffers skel:%p data:%p", allocation, skel, data);
+	pw_log_debug("%p: layout buffers skel:%p data:%p buffers:%p",
+			allocation, skel, data, buffers);
 	spa_buffer_alloc_layout_array(&info, n_buffers, buffers, skel, data);
 
 	allocation->mem = m;
@@ -166,7 +171,7 @@ param_filter(struct pw_buffers *this,
 
 	for (iidx = 0;;) {
 	        spa_pod_builder_init(&ib, ibuf, sizeof(ibuf));
-		pw_log_debug(NAME" %p: input param %d id:%d", this, iidx, id);
+		pw_log_debug("%p: input param %d id:%d", this, iidx, id);
 		in_res = spa_node_port_enum_params_sync(in_port->node,
 						in_port->direction, in_port->port_id,
 						id, &iidx, NULL, &iparam, &ib);
@@ -182,11 +187,10 @@ param_filter(struct pw_buffers *this,
 				break;
 		}
 
-		if (pw_log_level_enabled(SPA_LOG_LEVEL_DEBUG) && iparam != NULL)
-			spa_debug_pod(2, NULL, iparam);
+		pw_log_pod(SPA_LOG_LEVEL_DEBUG, iparam);
 
 		for (oidx = 0;;) {
-			pw_log_debug(NAME" %p: output param %d id:%d", this, oidx, id);
+			pw_log_debug("%p: output param %d id:%d", this, oidx, id);
 			out_res = spa_node_port_enum_params_sync(out_port->node,
 						out_port->direction, out_port->port_id,
 						id, &oidx, iparam, &oparam, result);
@@ -195,8 +199,7 @@ param_filter(struct pw_buffers *this,
 			if (out_res < 1)
 				break;
 
-			if (pw_log_level_enabled(SPA_LOG_LEVEL_DEBUG))
-				spa_debug_pod(2, NULL, oparam);
+			pw_log_pod(SPA_LOG_LEVEL_DEBUG, oparam);
 			num++;
 		}
 		if (out_res == -ENOENT && iparam) {
@@ -212,7 +215,11 @@ param_filter(struct pw_buffers *this,
 	if (num == 0) {
 		if (out_res == -ENOENT && in_res == -ENOENT)
 			return 0;
-		return in_res < 0 ? in_res : out_res < 0 ? out_res : -EINVAL;
+		if (in_res < 0)
+			return in_res;
+		if (out_res < 0)
+			return out_res;
+		return -EINVAL;
 	}
 	return num;
 }
@@ -238,53 +245,57 @@ int pw_buffers_negotiate(struct pw_context *context, uint32_t flags,
 	uint8_t buffer[4096];
 	struct spa_pod_builder b = SPA_POD_BUILDER_INIT(buffer, sizeof(buffer));
 	uint32_t i, offset, n_params;
-	uint32_t max_buffers;
+	uint32_t max_buffers, blocks;
 	size_t minsize, stride, align;
-	uint32_t data_sizes[1];
-	int32_t data_strides[1];
-	uint32_t data_aligns[1];
-	uint32_t types, data_types[1];
+	uint32_t *data_sizes;
+	int32_t *data_strides;
+	uint32_t *data_aligns;
+	uint32_t types, *data_types;
 	struct port output = { outnode, SPA_DIRECTION_OUTPUT, out_port_id };
 	struct port input = { innode, SPA_DIRECTION_INPUT, in_port_id };
-	const char *str;
 	int res;
 
 	res = param_filter(result, &input, &output, SPA_PARAM_Buffers, &b);
-	if (res < 0)
+	if (res < 0) {
+		pw_context_debug_port_params(context, input.node, input.direction,
+				input.port_id, SPA_PARAM_Buffers, res,
+				"input param");
+		pw_context_debug_port_params(context, output.node, output.direction,
+				output.port_id, SPA_PARAM_Buffers, res,
+				"output param");
 		return res;
+	}
 	n_params = res;
 	if ((res = param_filter(result, &input, &output, SPA_PARAM_Meta, &b)) > 0)
 		n_params += res;
 
 	params = alloca(n_params * sizeof(struct spa_pod *));
 	for (i = 0, offset = 0; i < n_params; i++) {
-		params[i] = SPA_MEMBER(buffer, offset, struct spa_pod);
+		params[i] = SPA_PTROFF(buffer, offset, struct spa_pod);
 		spa_pod_fixate(params[i]);
-		pw_log_debug(NAME" %p: fixated param %d:", result, i);
-		if (pw_log_level_enabled(SPA_LOG_LEVEL_DEBUG))
-			spa_debug_pod(2, NULL, params[i]);
+		pw_log_debug("%p: fixated param %d:", result, i);
+		pw_log_pod(SPA_LOG_LEVEL_DEBUG, params[i]);
 		offset += SPA_ROUND_UP_N(SPA_POD_SIZE(params[i]), 8);
 	}
 
-	max_buffers = context->defaults.link_max_buffers;
+	max_buffers = context->settings.link_max_buffers;
 
-	if ((str = pw_properties_get(context->properties, PW_KEY_CPU_MAX_ALIGN)) != NULL)
-		align = pw_properties_parse_int(str);
-	else
-		align = MAX_ALIGN;
+	align = pw_properties_get_uint32(context->properties, PW_KEY_CPU_MAX_ALIGN, MAX_ALIGN);
 
 	minsize = stride = 0;
 	types = SPA_ID_INVALID; /* bitmask of allowed types */
+	blocks = 1;
 
 	param = find_param(params, n_params, SPA_TYPE_OBJECT_ParamBuffers);
 	if (param) {
 		uint32_t qmax_buffers = max_buffers,
 		    qminsize = minsize, qstride = stride, qalign = align;
-		uint32_t qtypes = types;
+		uint32_t qtypes = types, qblocks = blocks;
 
 		spa_pod_parse_object(param,
 			SPA_TYPE_OBJECT_ParamBuffers, NULL,
 			SPA_PARAM_BUFFERS_buffers,  SPA_POD_OPT_Int(&qmax_buffers),
+			SPA_PARAM_BUFFERS_blocks,   SPA_POD_OPT_Int(&qblocks),
 			SPA_PARAM_BUFFERS_size,     SPA_POD_OPT_Int(&qminsize),
 			SPA_PARAM_BUFFERS_stride,   SPA_POD_OPT_Int(&qstride),
 			SPA_PARAM_BUFFERS_align,    SPA_POD_OPT_Int(&qalign),
@@ -293,16 +304,17 @@ int pw_buffers_negotiate(struct pw_context *context, uint32_t flags,
 		max_buffers =
 		    qmax_buffers == 0 ? max_buffers : SPA_MIN(qmax_buffers,
 						      max_buffers);
+		blocks = SPA_CLAMP(qblocks, blocks, MAX_BLOCKS);
 		minsize = SPA_MAX(minsize, qminsize);
 		stride = SPA_MAX(stride, qstride);
 		align = SPA_MAX(align, qalign);
 		types = qtypes;
 
-		pw_log_debug(NAME" %p: %d %d %d %d %d -> %zd %zd %d %zd %d", result,
-				qminsize, qstride, qmax_buffers, qalign, qtypes,
-				minsize, stride, max_buffers, align, types);
+		pw_log_debug("%p: %d %d %d %d %d %d -> %d %zd %zd %d %zd %d", result,
+				qblocks, qminsize, qstride, qmax_buffers, qalign, qtypes,
+				blocks, minsize, stride, max_buffers, align, types);
 	} else {
-		pw_log_warn(NAME" %p: no buffers param", result);
+		pw_log_warn("%p: no buffers param", result);
 		minsize = 8192;
 		max_buffers = 2;
 	}
@@ -310,21 +322,28 @@ int pw_buffers_negotiate(struct pw_context *context, uint32_t flags,
 	if (SPA_FLAG_IS_SET(flags, PW_BUFFERS_FLAG_NO_MEM))
 		minsize = 0;
 
-	data_sizes[0] = minsize;
-	data_strides[0] = stride;
-	data_aligns[0] = align;
-	data_types[0] = types;
+	data_sizes = alloca(sizeof(uint32_t) * blocks);
+	data_strides = alloca(sizeof(int32_t) * blocks);
+	data_aligns = alloca(sizeof(uint32_t) * blocks);
+	data_types = alloca(sizeof(uint32_t) * blocks);
+
+	for (i = 0; i < blocks; i++) {
+		data_sizes[i] = minsize;
+		data_strides[i] = stride;
+		data_aligns[i] = align;
+		data_types[i] = types;
+	}
 
 	if ((res = alloc_buffers(context->pool,
 				 max_buffers,
 				 n_params,
 				 params,
-				 1,
+				 blocks,
 				 data_sizes, data_strides,
 				 data_aligns, data_types,
 				 flags,
 				 result)) < 0) {
-		pw_log_error(NAME" %p: can't alloc buffers: %s", result, spa_strerror(res));
+		pw_log_error("%p: can't alloc buffers: %s", result, spa_strerror(res));
 	}
 
 	return res;
@@ -333,6 +352,7 @@ int pw_buffers_negotiate(struct pw_context *context, uint32_t flags,
 SPA_EXPORT
 void pw_buffers_clear(struct pw_buffers *buffers)
 {
+	pw_log_debug("%p: clear %d buffers:%p", buffers, buffers->n_buffers, buffers->buffers);
 	if (buffers->mem)
 		pw_memblock_unref(buffers->mem);
 	free(buffers->buffers);
