@@ -52,6 +52,9 @@
 
 #define SUPPORTLIB	"support/libspa-support"
 
+PW_LOG_TOPIC_EXTERN(log_context);
+#define PW_LOG_TOPIC_DEFAULT log_context
+
 static char *prgname;
 
 static struct spa_i18n *_pipewire_i18n = NULL;
@@ -81,17 +84,19 @@ struct support {
 	char **categories;
 	const char *plugin_dir;
 	const char *support_lib;
-	struct registry *registry;
+	struct registry registry;
 	char *i18n_domain;
 	struct spa_interface i18n_iface;
 	struct spa_support support[MAX_SUPPORT];
 	uint32_t n_support;
+	unsigned int initialized:1;
 	unsigned int in_valgrind:1;
 	unsigned int no_color:1;
 	unsigned int no_config:1;
 };
 
-static struct registry global_registry;
+static pthread_mutex_t init_lock = PTHREAD_MUTEX_INITIALIZER;
+static pthread_mutex_t support_lock = PTHREAD_MUTEX_INITIALIZER;
 static struct support global_support;
 
 static struct plugin *
@@ -202,37 +207,15 @@ static void unref_handle(struct handle *handle)
 	if (--handle->ref == 0) {
 		spa_list_remove(&handle->link);
 		pw_log_debug("clear handle '%s'", handle->factory_name);
+		pthread_mutex_unlock(&support_lock);
 		spa_handle_clear(&handle->handle);
+		pthread_mutex_lock(&support_lock);
 		unref_plugin(handle->plugin);
 		free(handle->factory_name);
 		free(handle);
 	}
 }
 
-static void configure_debug(struct support *support, const char *str)
-{
-	char **level;
-	int n_tokens;
-
-	level = pw_split_strv(str, ":", INT_MAX, &n_tokens);
-	if (n_tokens > 0) {
-		switch (level[0][0]) {
-			case 'X': pw_log_set_level(SPA_LOG_LEVEL_NONE); break;
-			case 'E': pw_log_set_level(SPA_LOG_LEVEL_ERROR); break;
-			case 'W': pw_log_set_level(SPA_LOG_LEVEL_WARN); break;
-			case 'I': pw_log_set_level(SPA_LOG_LEVEL_INFO); break;
-			case 'D': pw_log_set_level(SPA_LOG_LEVEL_DEBUG); break;
-			case 'T': pw_log_set_level(SPA_LOG_LEVEL_TRACE); break;
-			default:  pw_log_set_level(atoi(level[0])); break;
-		}
-	}
-
-	if (n_tokens > 1)
-		support->categories = pw_split_strv(level[1], ",", INT_MAX, &n_tokens);
-
-	pw_free_strv(level);
-}
-
 SPA_EXPORT
 uint32_t pw_get_support(struct spa_support *support, uint32_t max_support)
 {
@@ -242,8 +225,7 @@ uint32_t pw_get_support(struct spa_support *support, uint32_t max_support)
 	return n;
 }
 
-SPA_EXPORT
-struct spa_handle *pw_load_spa_handle(const char *lib,
+static struct spa_handle *load_spa_handle(const char *lib,
 		const char *factory_name,
 		const struct spa_dict *info,
 		uint32_t n_support,
@@ -271,13 +253,15 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 	res = -ENOENT;
 
 	while ((p = pw_split_walk(sup->plugin_dir, ":", &len, &state))) {
-		if ((plugin = open_plugin(sup->registry, p, len, lib)) != NULL)
+		if ((plugin = open_plugin(&sup->registry, p, len, lib)) != NULL)
 			break;
 		res = -errno;
 	}
 	if (plugin == NULL)
 		goto error_out;
 
+	pthread_mutex_unlock(&support_lock);
+
 	factory = find_factory(plugin, factory_name);
 	if (factory == NULL) {
 		res = -errno;
@@ -298,6 +282,7 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 		goto error_free_handle;
 	}
 
+	pthread_mutex_lock(&support_lock);
 	handle->ref = 1;
 	handle->plugin = plugin;
 	handle->factory_name = strdup(factory_name);
@@ -308,15 +293,30 @@ struct spa_handle *pw_load_spa_handle(const char *lib,
 error_free_handle:
 	free(handle);
 error_unref_plugin:
+	pthread_mutex_lock(&support_lock);
 	unref_plugin(plugin);
 error_out:
 	errno = -res;
 	return NULL;
 }
 
+SPA_EXPORT
+struct spa_handle *pw_load_spa_handle(const char *lib,
+		const char *factory_name,
+		const struct spa_dict *info,
+		uint32_t n_support,
+		const struct spa_support support[])
+{
+	struct spa_handle *handle;
+	pthread_mutex_lock(&support_lock);
+	handle = load_spa_handle(lib, factory_name, info, n_support, support);
+	pthread_mutex_unlock(&support_lock);
+	return handle;
+}
+
 static struct handle *find_handle(struct spa_handle *handle)
 {
-	struct registry *registry = global_support.registry;
+	struct registry *registry = &global_support.registry;
 	struct plugin *p;
 	struct handle *h;
 
@@ -333,13 +333,16 @@ SPA_EXPORT
 int pw_unload_spa_handle(struct spa_handle *handle)
 {
 	struct handle *h;
+	int res = 0;
 
+	pthread_mutex_lock(&support_lock);
 	if ((h = find_handle(handle)) == NULL)
-		return -ENOENT;
+		res = -ENOENT;
+	else
+		unref_handle(h);
+	pthread_mutex_unlock(&support_lock);
 
-	unref_handle(h);
-
-	return 0;
+	return res;
 }
 
 static void *add_interface(struct support *support,
@@ -351,17 +354,24 @@ static void *add_interface(struct support *support,
 	void *iface = NULL;
 	int res = -ENOENT;
 
-	handle = pw_load_spa_handle(support->support_lib,
+	handle = load_spa_handle(support->support_lib,
 			factory_name, info,
 			support->n_support, support->support);
+	if (handle == NULL)
+		return NULL;
 
-	if (handle == NULL ||
-	    (res = spa_handle_get_interface(handle, type, &iface)) < 0) {
-			pw_log_error("can't get %s interface %d", type, res);
-	} else {
-		support->support[support->n_support++] =
-			SPA_SUPPORT_INIT(type, iface);
+	pthread_mutex_unlock(&support_lock);
+	res = spa_handle_get_interface(handle, type, &iface);
+	pthread_mutex_lock(&support_lock);
+
+	if (res < 0 || iface == NULL) {
+		pw_log_error("can't get %s interface %d: %s", type, res,
+				spa_strerror(res));
+		return NULL;
 	}
+
+	support->support[support->n_support++] =
+		SPA_SUPPORT_INIT(type, iface);
 	return iface;
 }
 
@@ -376,6 +386,7 @@ int pw_set_domain(const char *domain)
 		return -errno;
 	return 0;
 }
+
 SPA_EXPORT
 const char *pw_get_domain(void)
 {
@@ -440,46 +451,122 @@ const char *pw_ngettext(const char *msgid, const char *msgid_plural, unsigned lo
 }
 
 #ifdef HAVE_SYSTEMD
-static struct spa_log *load_journal_logger(struct support *support)
+static struct spa_log *load_journal_logger(struct support *support,
+					   const struct spa_dict *info)
 {
 	struct spa_handle *handle;
 	void *iface = NULL;
 	int res = -ENOENT;
-	struct spa_dict info;
-	struct spa_dict_item items[1];
-	char level[32];
 	uint32_t i;
 
 	/* is the journal even available? */
 	if (access("/run/systemd/journal/socket", F_OK) != 0)
 		return NULL;
 
-	snprintf(level, sizeof(level), "%d", pw_log_level);
-	items[0] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LEVEL, level);
-	info = SPA_DICT_INIT(items, 1);
-
-	handle = pw_load_spa_handle("support/libspa-journal",
-				    SPA_NAME_SUPPORT_LOG, &info,
+	handle = load_spa_handle("support/libspa-journal",
+				    SPA_NAME_SUPPORT_LOG, info,
 				    support->n_support, support->support);
+	if (handle == NULL)
+		return NULL;
 
-	if (handle == NULL ||
-	    (res = spa_handle_get_interface(handle, SPA_TYPE_INTERFACE_Log, &iface)) < 0) {
-			pw_log_error("can't get log interface %d", res);
-	} else {
-		/* look for an existing logger, and
-		 * replace it with the journal logger */
-		for (i = 0; i < support->n_support; i++) {
-			if (spa_streq(support->support[i].type, SPA_TYPE_INTERFACE_Log)) {
-				support->support[i].data = iface;
-				break;
-			}
-		}
+	pthread_mutex_unlock(&support_lock);
+	res = spa_handle_get_interface(handle, SPA_TYPE_INTERFACE_Log, &iface);
+	pthread_mutex_lock(&support_lock);
+
+	if (res < 0 || iface == NULL) {
+		pw_log_error("can't get log interface %d: %s", res,
+				spa_strerror(res));
+		return NULL;
 	}
 
+	/* look for an existing logger, and
+	 * replace it with the journal logger */
+	for (i = 0; i < support->n_support; i++) {
+		if (spa_streq(support->support[i].type, SPA_TYPE_INTERFACE_Log)) {
+			support->support[i].data = iface;
+			break;
+		}
+	}
 	return (struct spa_log *) iface;
 }
 #endif
 
+static enum spa_log_level
+parse_log_level(const char *str)
+{
+	enum spa_log_level l = SPA_LOG_LEVEL_NONE;
+	switch(str[0]) {
+		case 'X': l = SPA_LOG_LEVEL_NONE; break;
+		case 'E': l = SPA_LOG_LEVEL_ERROR; break;
+		case 'W': l = SPA_LOG_LEVEL_WARN; break;
+		case 'I': l = SPA_LOG_LEVEL_INFO; break;
+		case 'D': l = SPA_LOG_LEVEL_DEBUG; break;
+		case 'T': l = SPA_LOG_LEVEL_TRACE; break;
+		default:
+			  l = atoi(str);
+			  break;
+	}
+	return l;
+}
+
+static char *
+parse_pw_debug_env(void)
+{
+	const char *str;
+	char **tokens;
+	int n_tokens;
+	size_t slen;
+	char json[1024] = {0};
+	char *pos = json;
+	char *end = pos + sizeof(json) - 1;
+
+	str = getenv("PIPEWIRE_DEBUG");
+
+	if (!str || (slen = strlen(str)) == 0)
+		return NULL;
+
+	/* String format is PIPEWIRE_DEBUG=<glob>:<level>[,<glob>:<level>,...],
+	 * converted into [{ conn.* = 0}, {glob = level}, {glob = level}, ....] ,
+	 * with the connection namespace disabled by default.
+	 */
+	pos += spa_scnprintf(pos, end - pos, "[ { conn.* = %d },", SPA_LOG_LEVEL_NONE);
+
+	/* We only have single-digit log levels, so any single-character
+	 * string is of the form PIPEWIRE_DEBUG=<N> */
+	if (slen == 1) {
+		pw_log_set_level(parse_log_level(str));
+		goto out;
+	}
+
+	tokens = pw_split_strv(str, ",", INT_MAX, &n_tokens);
+	if (n_tokens > 0) {
+		int i;
+		for (i = 0; i < n_tokens; i++) {
+			int n_tok;
+			char **tok;
+			char *pattern;
+			enum spa_log_level lvl;
+
+			tok = pw_split_strv(tokens[i], ":", 2, &n_tok);
+			if (n_tok == 2) {
+				pattern = tok[0];
+				lvl = parse_log_level(tok[1]);
+
+				pos += spa_scnprintf(pos, end - pos, "{ %s = %d },",
+						     pattern, lvl);
+			} else {
+				pw_log_warn("Ignoring invalid format in PIPEWIRE_DEBUG: '%s'\n", tokens[i]);
+			}
+
+			pw_free_strv(tok);
+		}
+	}
+	pw_free_strv(tokens);
+out:
+	pos += spa_scnprintf(pos, end - pos, "]");
+	return strdup(json);
+}
+
 /** Initialize PipeWire
  *
  * \param argc pointer to argc
@@ -495,27 +582,26 @@ SPA_EXPORT
 void pw_init(int *argc, char **argv[])
 {
 	const char *str;
-	struct spa_dict_item items[5];
+	struct spa_dict_item items[6];
 	uint32_t n_items;
 	struct spa_dict info;
 	struct support *support = &global_support;
 	struct spa_log *log;
 	char level[32];
 
-	if (support->registry != NULL)
-		return;
+	pthread_mutex_lock(&init_lock);
+	if (support->initialized)
+		goto done;
 
+	pthread_mutex_lock(&support_lock);
 	support->in_valgrind = RUNNING_ON_VALGRIND;
 
-	if ((str = getenv("NO_COLOR")) != NULL)
+	if (getenv("NO_COLOR") != NULL)
 		support->no_color = true;
 
 	if ((str = getenv("PIPEWIRE_NO_CONFIG")) != NULL)
 		support->no_config = pw_properties_parse_bool(str);
 
-	if ((str = getenv("PIPEWIRE_DEBUG")) != NULL)
-		configure_debug(support, str);
-
 	init_i18n(support);
 
 	if ((str = getenv("SPA_PLUGIN_DIR")) == NULL)
@@ -526,19 +612,23 @@ void pw_init(int *argc, char **argv[])
 		str = SUPPORTLIB;
 	support->support_lib = str;
 
-	spa_list_init(&global_registry.plugins);
-	support->registry = &global_registry;
+	spa_list_init(&support->registry.plugins);
 
 	if (pw_log_is_default()) {
+		char *patterns = NULL;
+
 		n_items = 0;
 		if (!support->no_color)
 			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_COLORS, "true");
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_TIMESTAMP, "true");
-		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LINE, "true");
+		if ((str = getenv("PIPEWIRE_LOG_LINE")) == NULL || spa_atob(str))
+			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LINE, "true");
 		snprintf(level, sizeof(level), "%d", pw_log_level);
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_LEVEL, level);
 		if ((str = getenv("PIPEWIRE_LOG")) != NULL)
 			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_FILE, str);
+		if ((patterns = parse_pw_debug_env()) != NULL)
+			items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_LOG_PATTERNS, patterns);
 		info = SPA_DICT_INIT(items, n_items);
 
 		log = add_interface(support, SPA_NAME_SUPPORT_LOG, SPA_TYPE_INTERFACE_Log, &info);
@@ -547,16 +637,19 @@ void pw_init(int *argc, char **argv[])
 
 #ifdef HAVE_SYSTEMD
 		if ((str = getenv("PIPEWIRE_LOG_SYSTEMD")) == NULL || spa_atob(str)) {
-			log = load_journal_logger(support);
+			log = load_journal_logger(support, &info);
 			if (log)
 				pw_log_set(log);
 		}
 #endif
+		free(patterns);
 	} else {
 		support->support[support->n_support++] =
 			SPA_SUPPORT_INIT(SPA_TYPE_INTERFACE_Log, pw_log_get());
 	}
 
+	pw_log_init();
+
 	n_items = 0;
 	if ((str = getenv("PIPEWIRE_CPU")))
 		items[n_items++] = SPA_DICT_ITEM_INIT(SPA_KEY_CPU_FORCE, str);
@@ -569,15 +662,21 @@ void pw_init(int *argc, char **argv[])
 	add_i18n(support);
 
 	pw_log_info("version %s", pw_get_library_version());
+	support->initialized = true;
+	pthread_mutex_unlock(&support_lock);
+done:
+	pthread_mutex_unlock(&init_lock);
 }
 
 SPA_EXPORT
 void pw_deinit(void)
 {
 	struct support *support = &global_support;
-	struct registry *registry = &global_registry;
+	struct registry *registry = &support->registry;
 	struct plugin *p;
 
+	pthread_mutex_lock(&init_lock);
+	pthread_mutex_lock(&support_lock);
 	pw_log_set(NULL);
 	spa_list_consume(p, &registry->plugins, link) {
 		struct handle *h;
@@ -589,7 +688,8 @@ void pw_deinit(void)
 	pw_free_strv(support->categories);
 	free(support->i18n_domain);
 	spa_zero(global_support);
-	spa_zero(global_registry);
+	pthread_mutex_unlock(&support_lock);
+	pthread_mutex_unlock(&init_lock);
 
 }
 
