@@ -25,11 +25,16 @@
 #include <stdio.h>
 #include <stdarg.h>
 #include <spa/utils/json.h>
+#include <spa/utils/string.h>
 
 #include "pipewire/array.h"
+#include "pipewire/log.h"
 #include "pipewire/utils.h"
 #include "pipewire/properties.h"
 
+PW_LOG_TOPIC_EXTERN(log_properties);
+#define PW_LOG_TOPIC_DEFAULT log_properties
+
 /** \cond */
 struct properties {
 	struct pw_properties this;
@@ -92,8 +97,6 @@ static struct properties *properties_new(int prealloc)
  * \param key a first key
  * \param ... value and more keys NULL terminated
  * \return a newly allocated properties object
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 struct pw_properties *pw_properties_new(const char *key, ...)
@@ -122,8 +125,6 @@ struct pw_properties *pw_properties_new(const char *key, ...)
  *
  * \param dict a dictionary. keys and values are copied
  * \return a new properties object
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 struct pw_properties *pw_properties_new_dict(const struct spa_dict *dict)
@@ -145,6 +146,14 @@ struct pw_properties *pw_properties_new_dict(const struct spa_dict *dict)
 	return &impl->this;
 }
 
+/** Update the properties from the given string, overwriting any
+ * existing keys with the new values from \a str.
+ *
+ * \a str should be a whitespace separated list of key=value
+ * strings or a json object, see pw_properties_new_string().
+ *
+ * \return The number of properties added or updated
+ */
 SPA_EXPORT
 int pw_properties_update_string(struct pw_properties *props, const char *str, size_t size)
 {
@@ -164,8 +173,6 @@ int pw_properties_update_string(struct pw_properties *props, const char *str, si
 		if ((len = spa_json_next(&it[1], &value)) <= 0)
 			break;
 
-		if (key[0] == '#')
-			continue;
 		if (spa_json_is_null(value, len))
 			val = NULL;
 		else {
@@ -183,13 +190,11 @@ int pw_properties_update_string(struct pw_properties *props, const char *str, si
 
 /** Make a new properties object from the given str
  *
- * \a str should be a whitespace separated list of key=value
+ * \a object should be a whitespace separated list of key=value
  * strings or a json object.
  *
- * \param args a property description
+ * \param object a property description
  * \return a new properties object
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 struct pw_properties *
@@ -216,8 +221,6 @@ error:
  *
  * \param properties properties to copy
  * \return a new properties object
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 struct pw_properties *pw_properties_copy(const struct pw_properties *properties)
@@ -227,16 +230,14 @@ struct pw_properties *pw_properties_copy(const struct pw_properties *properties)
 
 /** Copy multiple keys from one property to another
  *
- * \param src properties to copy from
- * \param dst properties to copy to
+ * \param props properties to copy to
+ * \param dict properties to copy from
  * \param keys a NULL terminated list of keys to copy
  * \return the number of keys changed in \a dest
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 int pw_properties_update_keys(struct pw_properties *props,
-		const struct spa_dict *dict, const char *keys[])
+		const struct spa_dict *dict, const char * const keys[])
 {
 	int i, changed = 0;
 	const char *str;
@@ -248,11 +249,11 @@ int pw_properties_update_keys(struct pw_properties *props,
 	return changed;
 }
 
-static bool has_key(const char *keys[], const char *key)
+static bool has_key(const char * const keys[], const char *key)
 {
 	int i;
 	for (i = 0; keys[i]; i++) {
-		if (strcmp(keys[i], key) == 0)
+		if (spa_streq(keys[i], key))
 			return true;
 	}
 	return false;
@@ -260,7 +261,7 @@ static bool has_key(const char *keys[], const char *key)
 
 SPA_EXPORT
 int pw_properties_update_ignore(struct pw_properties *props,
-		const struct spa_dict *dict, const char *ignore[])
+		const struct spa_dict *dict, const char * const ignore[])
 {
 	const struct spa_dict_item *it;
 	int changed = 0;
@@ -275,8 +276,6 @@ int pw_properties_update_ignore(struct pw_properties *props,
 /** Clear a properties object
  *
  * \param properties properties to clear
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 void pw_properties_clear(struct pw_properties *properties)
@@ -298,8 +297,6 @@ void pw_properties_clear(struct pw_properties *properties)
  *
  * The properties in \a props are updated with \a dict. Keys in \a dict
  * with NULL values are removed from \a props.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 int pw_properties_update(struct pw_properties *props,
@@ -321,8 +318,6 @@ int pw_properties_update(struct pw_properties *props,
  * \return the number of added properties
  *
  * The properties from \a dict that are not yet in \a props are added.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 int pw_properties_add(struct pw_properties *props,
@@ -347,12 +342,10 @@ int pw_properties_add(struct pw_properties *props,
  *
  * The properties with \a keys from \a dict that are not yet
  * in \a props are added.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 int pw_properties_add_keys(struct pw_properties *props,
-		const struct spa_dict *dict, const char *keys[])
+		const struct spa_dict *dict, const char * const keys[])
 {
 	uint32_t i;
 	int added = 0;
@@ -370,13 +363,16 @@ int pw_properties_add_keys(struct pw_properties *props,
 /** Free a properties object
  *
  * \param properties the properties to free
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 void pw_properties_free(struct pw_properties *properties)
 {
-	struct properties *impl = SPA_CONTAINER_OF(properties, struct properties, this);
+	struct properties *impl;
+
+	if (properties == NULL)
+		return;
+
+	impl = SPA_CONTAINER_OF(properties, struct properties, this);
 	pw_properties_clear(properties);
 	pw_array_clear(&impl->items);
 	free(impl);
@@ -396,11 +392,12 @@ static int do_replace(struct pw_properties *properties, const char *key, char *v
 		if (value == NULL)
 			return 0;
 		add_func(properties, strdup(key), copy ? strdup(value) : value);
+		SPA_FLAG_CLEAR(properties->dict.flags, SPA_DICT_FLAG_SORTED);
 	} else {
 		struct spa_dict_item *item =
 		    pw_array_get_unchecked(&impl->items, index, struct spa_dict_item);
 
-		if (value && strcmp(item->value, value) == 0)
+		if (value && spa_streq(item->value, value))
 			goto exit_noupdate;
 
 		if (value == NULL) {
@@ -412,6 +409,7 @@ static int do_replace(struct pw_properties *properties, const char *key, char *v
 			item->value = last->value;
 			impl->items.size -= sizeof(struct spa_dict_item);
 			properties->dict.n_items--;
+			SPA_FLAG_CLEAR(properties->dict.flags, SPA_DICT_FLAG_SORTED);
 		} else {
 			free((char *) item->value);
 			item->value = copy ? strdup(value) : value;
@@ -436,8 +434,6 @@ exit_noupdate:
  * Set the property in \a properties with \a key to \a value. Any previous value
  * of \a key will be overwritten. When \a value is NULL, the key will be
  * removed.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 int pw_properties_set(struct pw_properties *properties, const char *key, const char *value)
@@ -469,8 +465,6 @@ int pw_properties_setva(struct pw_properties *properties,
  *
  * Set the property in \a properties with \a key to the value in printf style \a format
  * Any previous value of \a key will be overwritten.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 int pw_properties_setf(struct pw_properties *properties, const char *key, const char *format, ...)
@@ -492,8 +486,6 @@ int pw_properties_setf(struct pw_properties *properties, const char *key, const
  * \return the property for \a key or NULL when the key was not found
  *
  * Get the property in \a properties with \a key.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 const char *pw_properties_get(const struct pw_properties *properties, const char *key)
@@ -507,6 +499,137 @@ const char *pw_properties_get(const struct pw_properties *properties, const char
 	return pw_array_get_unchecked(&impl->items, index, struct spa_dict_item)->value;
 }
 
+/** Fetch a property as uint32_t.
+ *
+ * \param properties a \ref pw_properties
+ * \param key a key
+ * \param value set to the value of the property on success, otherwise left
+ * unmodified
+ * \return 0 on success or a negative errno otherwise
+ * \retval -ENOENT The property does not exist
+ * \retval -EINVAL The property is not in the expected format
+ */
+SPA_EXPORT
+int pw_properties_fetch_uint32(const struct pw_properties *properties, const char *key,
+			       uint32_t *value)
+{
+	const char *str = pw_properties_get(properties, key);
+	bool success;
+
+	if (!str)
+		return -ENOENT;
+
+	success = spa_atou32(str, value, 0);
+	if (SPA_UNLIKELY(!success))
+		pw_log_warn("Failed to parse \"%s\"=\"%s\" as int32", key, str);
+
+	return success ? 0 : -EINVAL;
+}
+
+/** Fetch a property as int32_t
+ *
+ * \param properties a \ref pw_properties
+ * \param key a key
+ * \param value set to the value of the property on success, otherwise left
+ * unmodified
+ * \return 0 on success or a negative errno otherwise
+ * \retval -ENOENT The property does not exist
+ * \retval -EINVAL The property is not in the expected format
+ */
+SPA_EXPORT
+int pw_properties_fetch_int32(const struct pw_properties *properties, const char *key,
+			      int32_t *value)
+{
+	const char *str = pw_properties_get(properties, key);
+	bool success;
+
+	if (!str)
+		return -ENOENT;
+
+	success = spa_atoi32(str, value, 0);
+	if (SPA_UNLIKELY(!success))
+		pw_log_warn("Failed to parse \"%s\"=\"%s\" as int32", key, str);
+
+	return success ? 0 : -EINVAL;
+}
+
+/** Fetch a property as uint64_t.
+ *
+ * \param properties a \ref pw_properties
+ * \param key a key
+ * \param value set to the value of the property on success, otherwise left
+ * unmodified
+ * \return 0 on success or a negative errno otherwise
+ * \retval -ENOENT The property does not exist
+ * \retval -EINVAL The property is not in the expected format
+ */
+SPA_EXPORT
+int pw_properties_fetch_uint64(const struct pw_properties *properties, const char *key,
+			       uint64_t *value)
+{
+	const char *str = pw_properties_get(properties, key);
+	bool success;
+
+	if (!str)
+		return -ENOENT;
+
+	success = spa_atou64(str, value, 0);
+	if (SPA_UNLIKELY(!success))
+		pw_log_warn("Failed to parse \"%s\"=\"%s\" as uint64", key, str);
+
+	return success ? 0 : -EINVAL;
+}
+
+/** Fetch a property as int64_t
+ *
+ * \param properties a \ref pw_properties
+ * \param key a key
+ * \param value set to the value of the property on success, otherwise left
+ * unmodified
+ * \return 0 on success or a negative errno otherwise
+ * \retval -ENOENT The property does not exist
+ * \retval -EINVAL The property is not in the expected format
+ */
+SPA_EXPORT
+int pw_properties_fetch_int64(const struct pw_properties *properties, const char *key,
+			      int64_t *value)
+{
+	const char *str = pw_properties_get(properties, key);
+	bool success;
+
+	if (!str)
+		return -ENOENT;
+
+	success = spa_atoi64(str, value, 0);
+	if (SPA_UNLIKELY(!success))
+		pw_log_warn("Failed to parse \"%s\"=\"%s\" as int64", key, str);
+
+	return success ? 0 : -EINVAL;
+}
+
+/** Fetch a property as boolean value
+ *
+ * \param properties a \ref pw_properties
+ * \param key a key
+ * \param value set to the value of the property on success, otherwise left
+ * unmodified
+ * \return 0 on success or a negative errno otherwise
+ * \retval -ENOENT The property does not exist
+ * \retval -EINVAL The property is not in the expected format
+ */
+SPA_EXPORT
+int pw_properties_fetch_bool(const struct pw_properties *properties, const char *key,
+			     bool *value)
+{
+	const char *str = pw_properties_get(properties, key);
+
+	if (!str)
+		return -ENOENT;
+
+	*value = spa_atob(str);
+	return 0;
+}
+
 /** Iterate property values
  *
  * \param properties a \ref pw_properties
@@ -517,8 +640,6 @@ const char *pw_properties_get(const struct pw_properties *properties, const char
  * to a pointer holding NULL to get the first element and will be updated
  * after each iteration. When NULL is returned, all elements have been
  * iterated.
- *
- * \memberof pw_properties
  */
 SPA_EXPORT
 const char *pw_properties_iterate(const struct pw_properties *properties, void **state)
@@ -538,3 +659,75 @@ const char *pw_properties_iterate(const struct pw_properties *properties, void *
 
 	return pw_array_get_unchecked(&impl->items, index, struct spa_dict_item)->key;
 }
+
+static int encode_string(FILE *f, const char *val)
+{
+	int len = 0;
+	len += fprintf(f, "\"");
+	while (*val) {
+		switch (*val) {
+		case '\n':
+			len += fprintf(f, "\\n");
+			break;
+		case '\r':
+			len += fprintf(f, "\\r");
+			break;
+		case '\b':
+			len += fprintf(f, "\\b");
+			break;
+		case '\t':
+			len += fprintf(f, "\\t");
+			break;
+		case '\f':
+			len += fprintf(f, "\\f");
+			break;
+		case '\\':
+		case '"':
+			len += fprintf(f, "\\%c", *val);
+		break;
+		default:
+			if (*val > 0 && *val < 0x20)
+				len += fprintf(f, "\\u%04x", *val);
+			else
+				len += fprintf(f, "%c", *val);
+			break;
+		}
+		val++;
+	}
+	len += fprintf(f, "\"");
+	return len-1;
+}
+
+SPA_EXPORT
+int pw_properties_serialize_dict(FILE *f, const struct spa_dict *dict, uint32_t flags)
+{
+	const struct spa_dict_item *it;
+	int count = 0;
+	char key[1024];
+
+	spa_dict_for_each(it, dict) {
+		size_t len = it->value ? strlen(it->value) : 0;
+
+		if (spa_json_encode_string(key, sizeof(key)-1, it->key) >= (int)sizeof(key)-1)
+			continue;
+
+		fprintf(f, "%s%s %s: ",
+				count == 0 ? "" : ",",
+				flags & PW_PROPERTIES_FLAG_NL ? "\n" : "",
+				key);
+
+		if (it->value == NULL) {
+			fprintf(f, "null");
+		} else if (spa_json_is_null(it->value, len) ||
+		    spa_json_is_float(it->value, len) ||
+		    spa_json_is_bool(it->value, len) ||
+		    spa_json_is_container(it->value, len) ||
+		    spa_json_is_string(it->value, len)) {
+			fprintf(f, "%s", it->value);
+		} else {
+			encode_string(f, it->value);
+		}
+		count++;
+	}
+	return count;
+}
