@@ -29,13 +29,13 @@
 extern "C" {
 #endif
 
-/** \page page_streams Media Streams
+/** \page page_streams Streams
  *
  * \section sec_overview Overview
  *
- * Media streams are used to exchange data with the PipeWire server. A
- * stream is a wrapper around a proxy for a \ref pw_client_node with
- * an adapter. This means the stream will automatically do conversion
+ * \ref pw_stream "Streams" are used to exchange data with the
+ * PipeWire server. A stream is a wrapper around a proxy for a pw_client_node
+ * with an adapter. This means the stream will automatically do conversion
  * to the type required by the server.
  *
  * Streams can be used to:
@@ -53,9 +53,8 @@ extern "C" {
  * \section sec_create Create
  *
  * Make a new stream with \ref pw_stream_new(). You will need to specify
- * a name for the stream and extra properties. You can use \ref
- * pw_fill_stream_properties() to get a basic set of properties for the
- * stream.
+ * a name for the stream and extra properties. The basic set of properties
+ * each stream must provide is filled in automatically.
  *
  * Once the stream is created, the state_changed event should be used to
  * track the state of the stream.
@@ -99,7 +98,8 @@ extern "C" {
  *
  * With the add_buffer event, a stream will be notified of a new buffer
  * that can be used for data transport. You can attach user_data to these
- * buffers.
+ * buffers. The buffers can only be used with the stream that emitted
+ * the add_buffer event.
  *
  * After the buffers are negotiated, the stream will transition to the
  * \ref PW_STREAM_STATE_PAUSED state.
@@ -116,19 +116,25 @@ extern "C" {
  *
  * \subsection ssec_consume Consume data
  *
- * The process event is emitted for each new buffer that can can be
+ * The process event is emitted for each new buffer that can be
  * consumed.
  *
  * \ref pw_stream_dequeue_buffer() should be used to get the data and
  * metadata of the buffer.
  *
- * When the buffer is no longer in use, call \ref pw_stream_queue_buffer()
+ * The buffer is owned by the stream and stays alive until the
+ * remove_buffer event is emitted or the stream is destroyed.
+ *
+ * When the buffer has been processed, call \ref pw_stream_queue_buffer()
  * to let PipeWire reuse the buffer.
  *
  * \subsection ssec_produce Produce data
  *
  * \ref pw_stream_dequeue_buffer() gives an empty buffer that can be filled.
  *
+ * The buffer is owned by the stream and stays alive until the
+ * remove_buffer event is emitted or the stream is destroyed.
+ *
  * Filled buffers should be queued with \ref pw_stream_queue_buffer().
  *
  * The process event is emitted when PipeWire has emptied a buffer that
@@ -138,21 +144,27 @@ extern "C" {
  *
  * Use \ref pw_stream_disconnect() to disconnect a stream after use.
  */
-/** \class pw_stream
+/** \defgroup pw_stream Stream
  *
- * \brief PipeWire stream object class
+ * \brief PipeWire stream objects
  *
  * The stream object provides a convenient way to send and
  * receive data streams from/to PipeWire.
  *
- * See also \ref page_streams and \ref page_context_api
+ * See also \ref page_streams and \ref api_pw_core
+ */
+
+/**
+ * \addtogroup pw_stream
+ * \{
  */
 struct pw_stream;
 
 #include <spa/buffer/buffer.h>
 #include <spa/param/param.h>
+#include <spa/pod/command.h>
 
-/** \enum pw_stream_state The state of a stream \memberof pw_stream */
+/** \enum pw_stream_state The state of a stream */
 enum pw_stream_state {
 	PW_STREAM_STATE_ERROR = -1,		/**< the stream is in error */
 	PW_STREAM_STATE_UNCONNECTED = 0,	/**< unconnected */
@@ -179,7 +191,7 @@ struct pw_stream_control {
 	uint32_t max_values;		/**< max values that can be set on this control */
 };
 
-/** A time structure \memberof pw_stream */
+/** A time structure */
 struct pw_time {
 	int64_t now;			/**< the monotonic time in nanoseconds */
 	struct spa_fraction rate;	/**< the rate of \a ticks and delay */
@@ -193,12 +205,12 @@ struct pw_time {
 					  *  currently queued */
 };
 
-#include <pipewire/pipewire.h>
+#include <pipewire/port.h>
 
 /** Events for a stream. These events are always called from the mainloop
  * unless explicitly documented otherwise. */
 struct pw_stream_events {
-#define PW_VERSION_STREAM_EVENTS	0
+#define PW_VERSION_STREAM_EVENTS	1
 	uint32_t version;
 
 	void (*destroy) (void *data);
@@ -228,12 +240,14 @@ struct pw_stream_events {
 	/** The stream is drained */
         void (*drained) (void *data);
 
+	/** A command notify, Since 0.3.39:1 */
+	void (*command) (void *data, const struct spa_command *command);
 };
 
-/** Convert a stream state to a readable string \memberof pw_stream */
+/** Convert a stream state to a readable string */
 const char * pw_stream_state_as_string(enum pw_stream_state state);
 
-/** \enum pw_stream_flags Extra flags that can be used in \ref pw_stream_connect() \memberof pw_stream */
+/** \enum pw_stream_flags Extra flags that can be used in \ref pw_stream_connect() */
 enum pw_stream_flags {
 	PW_STREAM_FLAG_NONE = 0,			/**< no flags */
 	PW_STREAM_FLAG_AUTOCONNECT	= (1 << 0),	/**< try to automatically connect
@@ -256,7 +270,7 @@ enum pw_stream_flags {
 							  *  data of the buffer should be set */
 };
 
-/** Create a new unconneced \ref pw_stream \memberof pw_stream
+/** Create a new unconneced \ref pw_stream
  * \return a newly allocated \ref pw_stream */
 struct pw_stream *
 pw_stream_new(struct pw_core *core,		/**< a \ref pw_core */
@@ -270,7 +284,7 @@ pw_stream_new_simple(struct pw_loop *loop,	/**< a \ref pw_loop to use */
 		     const struct pw_stream_events *events,	/**< stream events */
 		     void *data					/**< data passed to events */);
 
-/** Destroy a stream \memberof pw_stream */
+/** Destroy a stream */
 void pw_stream_destroy(struct pw_stream *stream);
 
 void pw_stream_add_listener(struct pw_stream *stream,
@@ -288,7 +302,7 @@ const struct pw_properties *pw_stream_get_properties(struct pw_stream *stream);
 
 int pw_stream_update_properties(struct pw_stream *stream, const struct spa_dict *dict);
 
-/** Connect a stream for input or output on \a port_path. \memberof pw_stream
+/** Connect a stream for input or output on \a port_path.
  * \return 0 on success < 0 on error.
  *
  * You should connect to the process event and use pw_stream_dequeue_buffer()
@@ -305,20 +319,21 @@ pw_stream_connect(struct pw_stream *stream,		/**< a \ref pw_stream */
 							  *  formats. */
 		  uint32_t n_params			/**< number of items in \a params */);
 
-/** Get the node ID of the stream. \memberof pw_stream
+/** Get the node ID of the stream.
  * \return node ID. */
 uint32_t
 pw_stream_get_node_id(struct pw_stream *stream);
 
-/** Disconnect \a stream \memberof pw_stream */
+/** Disconnect \a stream  */
 int pw_stream_disconnect(struct pw_stream *stream);
 
 /** Set the stream in error state */
 int pw_stream_set_error(struct pw_stream *stream,	/**< a \ref pw_stream */
 			int res,			/**< a result code */
-			const char *error, ...		/**< an error message */) SPA_PRINTF_FUNC(3, 4);
+			const char *error,		/**< an error message */
+			...) SPA_PRINTF_FUNC(3, 4);
 
-/** Complete the negotiation process with result code \a res \memberof pw_stream
+/** Complete the negotiation process with result code \a res
  *
  * This function should be called after notification of the format.
 
@@ -331,26 +346,43 @@ pw_stream_update_params(struct pw_stream *stream,	/**< a \ref pw_stream */
 							  *  buffer allocation. */
 			uint32_t n_params		/**< number of elements in \a params */);
 
+/** Get control values */
+const struct pw_stream_control *pw_stream_get_control(struct pw_stream *stream, uint32_t id);
+
 /** Set control values */
 int pw_stream_set_control(struct pw_stream *stream, uint32_t id, uint32_t n_values, float *values, ...);
 
-/** Query the time on the stream \memberof pw_stream */
+/** Query the time on the stream  */
 int pw_stream_get_time(struct pw_stream *stream, struct pw_time *time);
 
 /** Get a buffer that can be filled for playback streams or consumed
- * for capture streams.  */
+ * for capture streams. */
 struct pw_buffer *pw_stream_dequeue_buffer(struct pw_stream *stream);
 
 /** Submit a buffer for playback or recycle a buffer for capture. */
 int pw_stream_queue_buffer(struct pw_stream *stream, struct pw_buffer *buffer);
 
-/** Activate or deactivate the stream \memberof pw_stream */
+/** Activate or deactivate the stream */
 int pw_stream_set_active(struct pw_stream *stream, bool active);
 
 /** Flush a stream. When \a drain is true, the drained callback will
  * be called when all data is played or recorded */
 int pw_stream_flush(struct pw_stream *stream, bool drain);
 
+/** Check if the stream is driving. The stream needs to have the
+ * PW_STREAM_FLAG_DRIVER set. When the stream is driving,
+ * pw_stream_trigger_process() needs to be called when data is
+ * available (output) or needed (input). Since 0.3.34 */
+bool pw_stream_is_driving(struct pw_stream *stream);
+
+/** Trigger a push/pull on the stream. One iteration of the graph will
+ * scheduled and process() will be called. Since 0.3.34 */
+int pw_stream_trigger_process(struct pw_stream *stream);
+
+/**
+ * \}
+ */
+
 #ifdef __cplusplus
 }
 #endif
