@@ -32,20 +32,25 @@
 #include <alloca.h>
 #endif
 #include <getopt.h>
+#include <readline/readline.h>
+#include <readline/history.h>
 
 #define spa_debug(...) fprintf(stdout,__VA_ARGS__);fputc('\n', stdout)
 
 #include <spa/utils/result.h>
+#include <spa/utils/string.h>
 #include <spa/debug/pod.h>
 #include <spa/utils/keys.h>
 #include <spa/utils/json.h>
 #include <spa/pod/builder.h>
 
 #include <pipewire/impl.h>
+#include <pipewire/i18n.h>
 
-#include <extensions/session-manager.h>
+#include <pipewire/extensions/session-manager.h>
 
 static const char WHITESPACE[] = " \t";
+static char prompt[64];
 
 struct remote_data;
 
@@ -116,44 +121,21 @@ struct command {
 static int pw_split_ip(char *str, const char *delimiter, int max_tokens, char *tokens[])
 {
 	const char *state = NULL;
-	char *s;
-	size_t len;
+	char *s, *t;
+	size_t len, l2;
 	int n = 0;
 
-        s = (char *)pw_split_walk(str, delimiter, &len, &state);
-        while (s && n + 1 < max_tokens) {
+	s = (char *)pw_split_walk(str, delimiter, &len, &state);
+	while (s && n + 1 < max_tokens) {
+		t = (char*)pw_split_walk(str, delimiter, &l2, &state);
 		s[len] = '\0';
 		tokens[n++] = s;
-                s = (char*)pw_split_walk(str, delimiter, &len, &state);
-        }
-        if (s) {
-		tokens[n++] = s;
-        }
-        return n;
-}
-
-static struct pw_properties *parse_props(char *str)
-{
-	const char *state = NULL;
-	char *s, *p[3];
-	size_t len, n;
-	struct pw_properties *props = NULL;
-
-	while (true) {
-		s = (char *)pw_split_walk(str, WHITESPACE, &len, &state);
-		if (s == NULL)
-			break;
-
-		s[len] = '\0';
-		n = pw_split_ip(s, "=", 2, p);
-		if (n == 2) {
-			if (props == NULL)
-				props = pw_properties_new(p[0], p[1], NULL);
-			else
-				pw_properties_set(props, p[0], p[1]);
-		}
+		s = t;
+		len = l2;
 	}
-	return props;
+	if (s)
+		tokens[n++] = s;
+	return n;
 }
 
 static void print_properties(struct spa_dict *props, char mark, bool header)
@@ -220,10 +202,11 @@ static bool do_set_param(struct data *data, const char *cmd, char *args, char **
 static bool do_permissions(struct data *data, const char *cmd, char *args, char **error);
 static bool do_get_permissions(struct data *data, const char *cmd, char *args, char **error);
 static bool do_dump(struct data *data, const char *cmd, char *args, char **error);
+static bool do_quit(struct data *data, const char *cmd, char *args, char **error);
 
 #define DUMP_NAMES "Core|Module|Device|Node|Port|Factory|Client|Link|Session|Endpoint|EndpointStream"
 
-static struct command command_list[] = {
+static const struct command command_list[] = {
 	{ "help", "h", "Show this help", do_help },
 	{ "load-module", "lm", "Load a module. <module-name> [<module-arguments>]", do_load_module },
 	{ "unload-module", "um", "Unload a module. <module-var>", do_not_implemented },
@@ -244,8 +227,16 @@ static struct command command_list[] = {
 	{ "get-permissions", "gp", "Get permissions of a client <client-id>", do_get_permissions },
 	{ "dump", "D", "Dump objects in ways that are cleaner for humans to understand "
 		 "[short|deep|resolve|notype] [-sdrt] [all|"DUMP_NAMES"|<id>]", do_dump },
+	{ "quit", "q", "Quit", do_quit },
 };
 
+static bool do_quit(struct data *data, const char *cmd, char *args, char **error)
+{
+	pw_main_loop_quit(data->loop);
+	data->quit = true;
+	return true;
+}
+
 static bool do_help(struct data *data, const char *cmd, char *args, char **error)
 {
 	size_t i;
@@ -291,11 +282,10 @@ static void on_core_info(void *_data, const struct pw_core_info *info)
 		fprintf(stdout, "remote %d is named '%s'\n", rd->id, rd->name);
 }
 
-static void show_prompt(struct remote_data *rd)
+static void set_prompt(struct remote_data *rd)
 {
-	rd->data->monitoring = true;
-	fprintf(stdout, "%s>>", rd->name);
-	fflush(stdout);
+	snprintf(prompt, sizeof(prompt), "%s>> ", rd->name);
+	rl_set_prompt(prompt);
 }
 
 static void on_core_done(void *_data, uint32_t id, int seq)
@@ -304,10 +294,12 @@ static void on_core_done(void *_data, uint32_t id, int seq)
 	struct data *d = rd->data;
 
 	if (seq == rd->prompt_pending) {
-		if (d->interactive)
-			show_prompt(rd);
-		else
+		if (d->interactive) {
+			set_prompt(rd);
+			rd->data->monitoring = true;
+		} else {
 			pw_main_loop_quit(d->loop);
+		}
 	}
 }
 
@@ -378,8 +370,7 @@ static int destroy_global(void *obj, void *data)
 		return 0;
 
 	pw_map_remove(&global->rd->globals, global->id);
-	if (global->properties)
-		pw_properties_free(global->properties);
+	pw_properties_free(global->properties);
 	free(global->type);
 	free(global);
 	return 0;
@@ -953,8 +944,7 @@ static const struct pw_device_events device_events = {
 static void session_info_free(struct pw_session_info *info)
 {
 	free(info->params);
-	if (info->props)
-		pw_properties_free ((struct pw_properties *)info->props);
+	pw_properties_free ((struct pw_properties *)info->props);
 	free(info);
 }
 
@@ -977,8 +967,7 @@ static void session_event_info(void *object,
 			info->n_params * sizeof(struct spa_param_info));
 	}
 	if (update->change_mask & PW_ENDPOINT_CHANGE_MASK_PROPS) {
-		if (info->props)
-			pw_properties_free ((struct pw_properties *)info->props);
+		pw_properties_free ((struct pw_properties *)info->props);
 		info->props =
 			(struct spa_dict *) pw_properties_new_dict (update->props);
 	}
@@ -1002,8 +991,7 @@ static void endpoint_info_free(struct pw_endpoint_info *info)
 	free(info->name);
 	free(info->media_class);
 	free(info->params);
-	if (info->props)
-		pw_properties_free ((struct pw_properties *)info->props);
+	pw_properties_free ((struct pw_properties *)info->props);
 	free(info);
 }
 
@@ -1034,8 +1022,7 @@ static void endpoint_event_info(void *object,
 			info->n_params * sizeof(struct spa_param_info));
 	}
 	if (update->change_mask & PW_ENDPOINT_CHANGE_MASK_PROPS) {
-		if (info->props)
-			pw_properties_free ((struct pw_properties *)info->props);
+		pw_properties_free ((struct pw_properties *)info->props);
 		info->props =
 			(struct spa_dict *) pw_properties_new_dict (update->props);
 	}
@@ -1058,8 +1045,7 @@ static void endpoint_stream_info_free(struct pw_endpoint_stream_info *info)
 {
 	free(info->name);
 	free(info->params);
-	if (info->props)
-		pw_properties_free ((struct pw_properties *)info->props);
+	pw_properties_free ((struct pw_properties *)info->props);
 	free(info);
 }
 
@@ -1084,8 +1070,7 @@ static void endpoint_stream_event_info(void *object,
 			info->n_params * sizeof(struct spa_param_info));
 	}
 	if (update->change_mask & PW_ENDPOINT_STREAM_CHANGE_MASK_PROPS) {
-		if (info->props)
-			pw_properties_free ((struct pw_properties *)info->props);
+		pw_properties_free ((struct pw_properties *)info->props);
 		info->props =
 			(struct spa_dict *) pw_properties_new_dict (update->props);
 	}
@@ -1152,57 +1137,57 @@ static bool bind_global(struct remote_data *rd, struct global *global, char **er
 	struct proxy_data *pd;
 	struct pw_proxy *proxy;
 
-	if (strcmp(global->type, PW_TYPE_INTERFACE_Core) == 0) {
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Core)) {
 		events = &core_events;
 		client_version = PW_VERSION_CORE;
 		destroy = (pw_destroy_t) pw_core_info_free;
 		info_func = info_core;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Module) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Module)) {
 		events = &module_events;
 		client_version = PW_VERSION_MODULE;
 		destroy = (pw_destroy_t) pw_module_info_free;
 		info_func = info_module;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Device) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Device)) {
 		events = &device_events;
 		client_version = PW_VERSION_DEVICE;
 		destroy = (pw_destroy_t) pw_device_info_free;
 		info_func = info_device;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Node) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Node)) {
 		events = &node_events;
 		client_version = PW_VERSION_NODE;
 		destroy = (pw_destroy_t) pw_node_info_free;
 		info_func = info_node;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Port) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Port)) {
 		events = &port_events;
 		client_version = PW_VERSION_PORT;
 		destroy = (pw_destroy_t) pw_port_info_free;
 		info_func = info_port;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Factory) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Factory)) {
 		events = &factory_events;
 		client_version = PW_VERSION_FACTORY;
 		destroy = (pw_destroy_t) pw_factory_info_free;
 		info_func = info_factory;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Client) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Client)) {
 		events = &client_events;
 		client_version = PW_VERSION_CLIENT;
 		destroy = (pw_destroy_t) pw_client_info_free;
 		info_func = info_client;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Link) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Link)) {
 		events = &link_events;
 		client_version = PW_VERSION_LINK;
 		destroy = (pw_destroy_t) pw_link_info_free;
 		info_func = info_link;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Session) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Session)) {
 		events = &session_events;
 		client_version = PW_VERSION_SESSION;
 		destroy = (pw_destroy_t) session_info_free;
 		info_func = info_session;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_Endpoint) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Endpoint)) {
 		events = &endpoint_events;
 		client_version = PW_VERSION_ENDPOINT;
 		destroy = (pw_destroy_t) endpoint_info_free;
 		info_func = info_endpoint;
-	} else if (strcmp(global->type, PW_TYPE_INTERFACE_EndpointStream) == 0) {
+	} else if (spa_streq(global->type, PW_TYPE_INTERFACE_EndpointStream)) {
 		events = &endpoint_stream_events;
 		client_version = PW_VERSION_ENDPOINT_STREAM;
 		destroy = (pw_destroy_t) endpoint_stream_info_free;
@@ -1278,7 +1263,7 @@ static bool do_info(struct data *data, const char *cmd, char *args, char **error
 		*error = spa_aprintf("%s <object-id>|all", cmd);
 		return false;
 	}
-	if (strcmp(a[0], "all") == 0) {
+	if (spa_streq(a[0], "all")) {
 		pw_map_for_each(&rd->globals, do_global_info_all, NULL);
 	}
 	else {
@@ -1309,7 +1294,7 @@ static bool do_create_device(struct data *data, const char *cmd, char *args, cha
 		return false;
 	}
 	if (n == 2)
-		props = parse_props(a[1]);
+		props = pw_properties_new_string(a[1]);
 
 	proxy = pw_core_create_object(rd->core, a[0],
 					    PW_TYPE_INTERFACE_Device,
@@ -1317,6 +1302,8 @@ static bool do_create_device(struct data *data, const char *cmd, char *args, cha
 					    props ? &props->dict : NULL,
 					    sizeof(struct proxy_data));
 
+	pw_properties_free(props);
+
 	pd = pw_proxy_get_user_data(proxy);
 	pd->rd = rd;
 	pd->proxy = proxy;
@@ -1346,7 +1333,7 @@ static bool do_create_node(struct data *data, const char *cmd, char *args, char
 		return false;
 	}
 	if (n == 2)
-		props = parse_props(a[1]);
+		props = pw_properties_new_string(a[1]);
 
 	proxy = pw_core_create_object(rd->core, a[0],
 					    PW_TYPE_INTERFACE_Node,
@@ -1354,6 +1341,8 @@ static bool do_create_node(struct data *data, const char *cmd, char *args, char
 					    props ? &props->dict : NULL,
 					    sizeof(struct proxy_data));
 
+	pw_properties_free(props);
+
 	pd = pw_proxy_get_user_data(proxy);
 	pd->rd = rd;
 	pd->proxy = proxy;
@@ -1407,14 +1396,18 @@ static bool do_create_link(struct data *data, const char *cmd, char *args, char
 		return false;
 	}
 	if (n == 5)
-		props = parse_props(a[4]);
+		props = pw_properties_new_string(a[4]);
 	else
 		props = pw_properties_new(NULL, NULL);
 
-	pw_properties_set(props, PW_KEY_LINK_OUTPUT_NODE, a[0]);
-	pw_properties_set(props, PW_KEY_LINK_OUTPUT_PORT, a[1]);
-	pw_properties_set(props, PW_KEY_LINK_INPUT_NODE, a[2]);
-	pw_properties_set(props, PW_KEY_LINK_INPUT_PORT, a[3]);
+	if (!spa_streq(a[0], "-"))
+		pw_properties_set(props, PW_KEY_LINK_OUTPUT_NODE, a[0]);
+	if (!spa_streq(a[1], "-"))
+		pw_properties_set(props, PW_KEY_LINK_OUTPUT_PORT, a[1]);
+	if (!spa_streq(a[2], "-"))
+		pw_properties_set(props, PW_KEY_LINK_INPUT_NODE, a[2]);
+	if (!spa_streq(a[3], "-"))
+		pw_properties_set(props, PW_KEY_LINK_INPUT_PORT, a[3]);
 
 	proxy = (struct pw_proxy*)pw_core_create_object(rd->core,
 					  "link-factory",
@@ -1423,6 +1416,8 @@ static bool do_create_link(struct data *data, const char *cmd, char *args, char
 					  props ? &props->dict : NULL,
 					  sizeof(struct proxy_data));
 
+	pw_properties_free(props);
+
 	pd = pw_proxy_get_user_data(proxy);
 	pd->rd = rd;
 	pd->proxy = proxy;
@@ -1480,12 +1475,27 @@ static bool do_export_node(struct data *data, const char *cmd, char *args, char
 	return false;
 }
 
+static const struct spa_type_info *find_type_info(const struct spa_type_info *info, const char *name)
+{
+	while (info && info->name) {
+                if (spa_streq(info->name, name))
+                        return info;
+                if (spa_streq(spa_debug_type_short_name(info->name), name))
+                        return info;
+                if (info->type != 0 && info->type == (uint32_t)atoi(name))
+                        return info;
+                info++;
+        }
+        return NULL;
+}
+
 static bool do_enum_params(struct data *data, const char *cmd, char *args, char **error)
 {
 	struct remote_data *rd = data->current;
 	char *a[2];
-        int n;
+	int n;
 	uint32_t id, param_id;
+	const struct spa_type_info *ti;
 	struct global *global;
 
 	n = pw_split_ip(args, WHITESPACE, 2, a);
@@ -1495,7 +1505,12 @@ static bool do_enum_params(struct data *data, const char *cmd, char *args, char
 	}
 
 	id = atoi(a[0]);
-	param_id = atoi(a[1]);
+	ti = find_type_info(spa_type_param, a[1]);
+	if (ti == NULL) {
+		*error = spa_aprintf("%s: unknown param type: %s", cmd, a[1]);
+		return false;
+	}
+	param_id = ti->type;
 
 	global = pw_map_lookup(&rd->globals, id);
 	if (global == NULL) {
@@ -1507,16 +1522,16 @@ static bool do_enum_params(struct data *data, const char *cmd, char *args, char
 			return false;
 	}
 
-	if (strcmp(global->type, PW_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Node))
 		pw_node_enum_params((struct pw_node*)global->proxy, 0,
 			param_id, 0, 0, NULL);
-	else if (strcmp(global->type, PW_TYPE_INTERFACE_Port) == 0)
+	else if (spa_streq(global->type, PW_TYPE_INTERFACE_Port))
 		pw_port_enum_params((struct pw_port*)global->proxy, 0,
 			param_id, 0, 0, NULL);
-	else if (strcmp(global->type, PW_TYPE_INTERFACE_Device) == 0)
+	else if (spa_streq(global->type, PW_TYPE_INTERFACE_Device))
 		pw_device_enum_params((struct pw_device*)global->proxy, 0,
 			param_id, 0, 0, NULL);
-	else if (strcmp(global->type, PW_TYPE_INTERFACE_Endpoint) == 0)
+	else if (spa_streq(global->type, PW_TYPE_INTERFACE_Endpoint))
 		pw_endpoint_enum_params((struct pw_endpoint*)global->proxy, 0,
 			param_id, 0, 0, NULL);
 	else {
@@ -1527,20 +1542,6 @@ static bool do_enum_params(struct data *data, const char *cmd, char *args, char
 	return true;
 }
 
-static const struct spa_type_info *find_type_info(const struct spa_type_info *info, const char *name)
-{
-	while (info && info->name) {
-                if (strcmp(info->name, name) == 0)
-                        return info;
-                if (strcmp(spa_debug_type_short_name(info->name), name) == 0)
-                        return info;
-                if (info->type != 0 && info->type == (uint32_t)atoi(name))
-                        return info;
-                info++;
-        }
-        return NULL;
-}
-
 static int json_to_pod(struct spa_pod_builder *b, uint32_t id,
 		const struct spa_type_info *info, struct spa_json *iter, const char *value, int len)
 {
@@ -1550,8 +1551,9 @@ static int json_to_pod(struct spa_pod_builder *b, uint32_t id,
 	struct spa_json it[1];
 	int l, res;
 	const char *v;
+	uint32_t type;
 
-	if (spa_json_is_object(value, len)) {
+	if (spa_json_is_object(value, len) && info != NULL) {
 		if ((ti = spa_debug_type_find(NULL, info->parent)) == NULL)
 			return -EINVAL;
 
@@ -1562,26 +1564,33 @@ static int json_to_pod(struct spa_pod_builder *b, uint32_t id,
 			const struct spa_type_info *pi;
 			if ((l = spa_json_next(&it[0], &v)) <= 0)
 				break;
-			if ((pi = find_type_info(ti->values, key)) == NULL)
+			if ((pi = find_type_info(ti->values, key)) != NULL)
+				type = pi->type;
+			else if ((type = atoi(key)) == 0)
 				continue;
-			spa_pod_builder_prop(b, pi->type, 0);
+			spa_pod_builder_prop(b, type, 0);
 			if ((res = json_to_pod(b, id, pi, &it[0], v, l)) < 0)
 				return res;
 		}
 		spa_pod_builder_pop(b, &f[0]);
 	}
 	else if (spa_json_is_array(value, len)) {
-		spa_pod_builder_push_array(b, &f[0]);
+		if (info == NULL || info->parent == SPA_TYPE_Struct) {
+			spa_pod_builder_push_struct(b, &f[0]);
+		} else {
+			spa_pod_builder_push_array(b, &f[0]);
+			info = info->values;
+		}
 		spa_json_enter(iter, &it[0]);
 		while ((l = spa_json_next(&it[0], &v)) > 0)
-			if ((res = json_to_pod(b, id, info->values, &it[0], v, l)) < 0)
+			if ((res = json_to_pod(b, id, info, &it[0], v, l)) < 0)
 				return res;
 		spa_pod_builder_pop(b, &f[0]);
 	}
 	else if (spa_json_is_float(value, len)) {
 		float val = 0.0f;
 		spa_json_parse_float(value, len, &val);
-		switch (info->parent) {
+		switch (info ? info->parent : SPA_TYPE_Struct) {
 		case SPA_TYPE_Bool:
 			spa_pod_builder_bool(b, val >= 0.5f);
 			break;
@@ -1594,6 +1603,12 @@ static int json_to_pod(struct spa_pod_builder *b, uint32_t id,
 		case SPA_TYPE_Long:
 			spa_pod_builder_long(b, val);
 			break;
+		case SPA_TYPE_Struct:
+			if (spa_json_is_int(value, len))
+				spa_pod_builder_int(b, val);
+			else
+				spa_pod_builder_float(b, val);
+			break;
 		case SPA_TYPE_Float:
 			spa_pod_builder_float(b, val);
 			break;
@@ -1616,12 +1631,15 @@ static int json_to_pod(struct spa_pod_builder *b, uint32_t id,
 	else {
 		char *val = alloca(len+1);
 		spa_json_parse_string(value, len, val);
-		switch (info->parent) {
+		switch (info ? info->parent : SPA_TYPE_Struct) {
 		case SPA_TYPE_Id:
-			if ((ti = find_type_info(info ? info->values : info, val)) == NULL)
+			if ((ti = find_type_info(info->values, val)) != NULL)
+				type = ti->type;
+			else if ((type = atoi(val)) == 0)
 				return -EINVAL;
-			spa_pod_builder_id(b, ti->type);
+			spa_pod_builder_id(b, type);
 			break;
+		case SPA_TYPE_Struct:
 		case SPA_TYPE_String:
 			spa_pod_builder_string(b, val);
 			break;
@@ -1687,13 +1705,13 @@ static bool do_set_param(struct data *data, const char *cmd, char *args, char **
 	}
 	spa_debug_pod(0, NULL, pod);
 
-	if (strcmp(global->type, PW_TYPE_INTERFACE_Node) == 0)
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Node))
 		pw_node_set_param((struct pw_node*)global->proxy,
 				param_id, 0, pod);
-	else if (strcmp(global->type, PW_TYPE_INTERFACE_Device) == 0)
+	else if (spa_streq(global->type, PW_TYPE_INTERFACE_Device))
 		pw_device_set_param((struct pw_device*)global->proxy,
 				param_id, 0, pod);
-	else if (strcmp(global->type, PW_TYPE_INTERFACE_Endpoint) == 0)
+	else if (spa_streq(global->type, PW_TYPE_INTERFACE_Endpoint))
 		pw_endpoint_set_param((struct pw_endpoint*)global->proxy,
 				param_id, 0, pod);
 	else {
@@ -1725,7 +1743,7 @@ static bool do_permissions(struct data *data, const char *cmd, char *args, char
 		*error = spa_aprintf("%s: unknown global %d", cmd, id);
 		return false;
 	}
-	if (strcmp(global->type, PW_TYPE_INTERFACE_Client) != 0) {
+	if (!spa_streq(global->type, PW_TYPE_INTERFACE_Client)) {
 		*error = spa_aprintf("object %d is not a client", atoi(a[0]));
 		return false;
 	}
@@ -1765,7 +1783,7 @@ static bool do_get_permissions(struct data *data, const char *cmd, char *args, c
 		*error = spa_aprintf("%s: unknown global %d", cmd, id);
 		return false;
 	}
-	if (strcmp(global->type, PW_TYPE_INTERFACE_Client) != 0) {
+	if (!spa_streq(global->type, PW_TYPE_INTERFACE_Client)) {
 		*error = spa_aprintf("object %d is not a client", atoi(a[0]));
 		return false;
 	}
@@ -1825,27 +1843,27 @@ global_props(struct global *global)
 	if (!pd || !pd->info)
 		return NULL;
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Core))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Core))
 		return ((struct pw_core_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Module))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Module))
 		return ((struct pw_module_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Device))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Device))
 		return ((struct pw_device_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Node))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Node))
 		return ((struct pw_node_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Port))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Port))
 		return ((struct pw_port_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Factory))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Factory))
 		return ((struct pw_factory_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Client))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Client))
 		return ((struct pw_client_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Link))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Link))
 		return ((struct pw_link_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Session))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Session))
 		return ((struct pw_session_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Endpoint))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Endpoint))
 		return ((struct pw_endpoint_info *)pd->info)->props;
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_EndpointStream))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_EndpointStream))
 		return ((struct pw_endpoint_stream_info *)pd->info)->props;
 
 	return NULL;
@@ -1912,20 +1930,20 @@ children_of(struct remote_data *rd, uint32_t parent_id,
 		return -1;
 
 	/* supported combinations */
-	if (!strcmp(parent_type, PW_TYPE_INTERFACE_Device) &&
-	    !strcmp(child_type, PW_TYPE_INTERFACE_Node)) {
+	if (spa_streq(parent_type, PW_TYPE_INTERFACE_Device) &&
+	    spa_streq(child_type, PW_TYPE_INTERFACE_Node)) {
 		parent_key = PW_KEY_OBJECT_ID;
 		child_key = PW_KEY_DEVICE_ID;
-	} else if (!strcmp(parent_type, PW_TYPE_INTERFACE_Node) &&
-		   !strcmp(child_type, PW_TYPE_INTERFACE_Port)) {
+	} else if (spa_streq(parent_type, PW_TYPE_INTERFACE_Node) &&
+		   spa_streq(child_type, PW_TYPE_INTERFACE_Port)) {
 		parent_key = PW_KEY_OBJECT_ID;
 		child_key = PW_KEY_NODE_ID;
-	} else if (!strcmp(parent_type, PW_TYPE_INTERFACE_Module) &&
-		   !strcmp(child_type, PW_TYPE_INTERFACE_Factory)) {
+	} else if (spa_streq(parent_type, PW_TYPE_INTERFACE_Module) &&
+		   spa_streq(child_type, PW_TYPE_INTERFACE_Factory)) {
 		parent_key = PW_KEY_OBJECT_ID;
 		child_key = PW_KEY_MODULE_ID;
-	} else if (!strcmp(parent_type, PW_TYPE_INTERFACE_Factory) &&
-		   !strcmp(child_type, PW_TYPE_INTERFACE_Device)) {
+	} else if (spa_streq(parent_type, PW_TYPE_INTERFACE_Factory) &&
+		   spa_streq(child_type, PW_TYPE_INTERFACE_Device)) {
 		parent_key = PW_KEY_OBJECT_ID;
 		child_key = PW_KEY_FACTORY_ID;
 	} else
@@ -1958,7 +1976,7 @@ children_of(struct remote_data *rd, uint32_t parent_id,
 
 			global = item->data;
 
-			if (strcmp(global->type, child_type))
+			if (!spa_streq(global->type, child_type))
 				continue;
 
 			pd = pw_proxy_get_user_data(global->proxy);
@@ -1973,7 +1991,7 @@ children_of(struct remote_data *rd, uint32_t parent_id,
 			}
 
 			/* match? */
-			if (strcmp(parent_value, child_value))
+			if (!spa_streq(parent_value, child_value))
 				continue;
 
 			if (*children)
@@ -1997,7 +2015,7 @@ enum dump_flags {
 	is_notype = BIT(3)
 };
 
-static const char *dump_types[] = {
+static const char * const dump_types[] = {
 	PW_TYPE_INTERFACE_Core,
 	PW_TYPE_INTERFACE_Module,
 	PW_TYPE_INTERFACE_Device,
@@ -2019,7 +2037,7 @@ int dump_type_index(const char *type)
 		return -1;
 
 	for (i = 0; i < SPA_N_ELEMENTS(dump_types); i++) {
-		if (!strcmp(dump_types[i], type))
+		if (spa_streq(dump_types[i], type))
 			return (int)i;
 	}
 
@@ -2083,20 +2101,19 @@ dump_properties(struct data *data, struct global *global,
 				ind, item->key, item->value);
 
 		extra = NULL;
-		id = -1;
-		if (!strcmp(global->type, PW_TYPE_INTERFACE_Port) && !strcmp(item->key, PW_KEY_NODE_ID)) {
+		if (spa_streq(global->type, PW_TYPE_INTERFACE_Port) && spa_streq(item->key, PW_KEY_NODE_ID)) {
 			id = atoi(item->value);
 			if (id >= 0)
 				extra = obj_lookup(rd, id, PW_KEY_NODE_NAME);
-		} else if (!strcmp(global->type, PW_TYPE_INTERFACE_Factory) && !strcmp(item->key, PW_KEY_MODULE_ID)) {
+		} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Factory) && spa_streq(item->key, PW_KEY_MODULE_ID)) {
 			id = atoi(item->value);
 			if (id >= 0)
 				extra = obj_lookup(rd, id, PW_KEY_MODULE_NAME);
-		} else if (!strcmp(global->type, PW_TYPE_INTERFACE_Device) && !strcmp(item->key, PW_KEY_FACTORY_ID)) {
+		} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Device) && spa_streq(item->key, PW_KEY_FACTORY_ID)) {
 			id = atoi(item->value);
 			if (id >= 0)
 				extra = obj_lookup(rd, id, PW_KEY_FACTORY_NAME);
-		} else if (!strcmp(global->type, PW_TYPE_INTERFACE_Device) && !strcmp(item->key, PW_KEY_CLIENT_ID)) {
+		} else if (spa_streq(global->type, PW_TYPE_INTERFACE_Device) && spa_streq(item->key, PW_KEY_CLIENT_ID)) {
 			id = atoi(item->value);
 			if (id >= 0)
 				extra = obj_lookup(rd, id, PW_KEY_CLIENT_NAME);
@@ -2293,8 +2310,8 @@ dump_device(struct data *data, struct global *global,
 				api ? api : "",
 				api ? "\"" : "");
 
-		if (media_class && !strcmp(media_class, "Audio/Device") &&
-		    api && !strcmp(api, "alsa:pcm")) {
+		if (media_class && spa_streq(media_class, "Audio/Device") &&
+		    api && spa_streq(api, "alsa:pcm")) {
 
 			alsa_path = spa_dict_lookup(info->props, SPA_KEY_API_ALSA_PATH);
 			alsa_card_id = spa_dict_lookup(info->props, SPA_KEY_API_ALSA_CARD_ID);
@@ -2721,37 +2738,37 @@ dump(struct data *data, struct global *global,
 	if (!global)
 		return;
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Core))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Core))
 		dump_core(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Module))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Module))
 		dump_module(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Device))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Device))
 		dump_device(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Node))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Node))
 		dump_node(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Port))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Port))
 		dump_port(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Factory))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Factory))
 		dump_factory(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Client))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Client))
 		dump_client(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Link))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Link))
 		dump_link(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Session))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Session))
 		dump_session(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_Endpoint))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_Endpoint))
 		dump_endpoint(data, global, flags, level);
 
-	if (!strcmp(global->type, PW_TYPE_INTERFACE_EndpointStream))
+	if (spa_streq(global->type, PW_TYPE_INTERFACE_EndpointStream))
 		dump_endpoint_stream(data, global, flags, level);
 }
 
@@ -2773,17 +2790,17 @@ static bool do_dump(struct data *data, const char *cmd, char *args, char **error
 
 	a = aa;
 	while (n > 0 &&
-		(!strcmp(a[0], "short") ||
-		 !strcmp(a[0], "deep") ||
-		 !strcmp(a[0], "resolve") ||
-		 !strcmp(a[0], "notype"))) {
-		if (!strcmp(a[0], "short"))
+		(spa_streq(a[0], "short") ||
+		 spa_streq(a[0], "deep") ||
+		 spa_streq(a[0], "resolve") ||
+		 spa_streq(a[0], "notype"))) {
+		if (spa_streq(a[0], "short"))
 			flags |= is_short;
-		else if (!strcmp(a[0], "deep"))
+		else if (spa_streq(a[0], "deep"))
 			flags |= is_deep;
-		else if (!strcmp(a[0], "resolve"))
+		else if (spa_streq(a[0], "resolve"))
 			flags |= is_resolve;
-		else if (!strcmp(a[0], "notype"))
+		else if (spa_streq(a[0], "notype"))
 			flags |= is_notype;
 		n--;
 		a++;
@@ -2806,7 +2823,7 @@ static bool do_dump(struct data *data, const char *cmd, char *args, char **error
 		a++;
 	}
 
-	if (n == 0 || !strcmp(a[0], "all")) {
+	if (n == 0 || spa_streq(a[0], "all")) {
 		type_mask = (1U << dump_type_count()) - 1;
 		flags &= ~is_notype;
 	} else {
@@ -2867,7 +2884,7 @@ usage:
 	return false;
 }
 
-static bool parse(struct data *data, char *buf, size_t size, char **error)
+static bool parse(struct data *data, char *buf, char **error)
 {
 	char *a[2];
 	int n;
@@ -2890,8 +2907,8 @@ static bool parse(struct data *data, char *buf, size_t size, char **error)
 	args = n > 1 ? a[1] : "";
 
 	for (i = 0; i < SPA_N_ELEMENTS(command_list); i++) {
-		if (!strcmp(command_list[i].name, cmd) ||
-		    !strcmp(command_list[i].alias, cmd)) {
+		if (spa_streq(command_list[i].name, cmd) ||
+		    spa_streq(command_list[i].alias, cmd)) {
 			return command_list[i].func(data, cmd, args, error);
 		}
 	}
@@ -2899,35 +2916,35 @@ static bool parse(struct data *data, char *buf, size_t size, char **error)
 	return false;
 }
 
-static void do_input(void *data, int fd, uint32_t mask)
+/* We need a global variable, readline doesn't have a closure arg */
+static struct data *readline_dataptr;
+
+static void readline_process_line(char *line)
 {
-	struct data *d = data;
-	char buf[4096], *error;
-	ssize_t r;
+	struct data *d = readline_dataptr;
+	char *error;
 
-	if (mask & SPA_IO_IN) {
-		while (true) {
-			r = read(fd, buf, sizeof(buf)-1);
-			if (r < 0) {
-				if (errno == EAGAIN)
-					continue;
-				perror("read");
-				r = 0;
-				break;
-			}
-			break;
-		}
-		if (r == 0) {
-			fprintf(stdout, "\n");
-			pw_main_loop_quit(d->loop);
-			return;
-		}
-		buf[r] = '\0';
+	if (!line)
+		line = strdup("quit");
 
-		if (!parse(d, buf, r, &error)) {
+	if (line[0] != '\0') {
+		add_history(line);
+		if (!parse(d, line, &error)) {
 			fprintf(stdout, "Error: \"%s\"\n", error);
 			free(error);
 		}
+	}
+	free(line);
+}
+
+static void do_input(void *data, int fd, uint32_t mask)
+{
+	struct data *d = data;
+
+	if (mask & SPA_IO_IN) {
+		readline_dataptr = d;
+		rl_callback_read_char();
+
 		if (d->current == NULL)
 			pw_main_loop_quit(d->loop);
 		else  {
@@ -2938,7 +2955,56 @@ static void do_input(void *data, int fd, uint32_t mask)
 	}
 }
 
-static void do_quit(void *data, int signal_number)
+static char *
+readline_match_command(const char *text, int state)
+{
+	static size_t idx;
+	static int len;
+
+	if (!state) {
+		idx = 0;
+		len = strlen(text);
+	}
+
+	while (idx < SPA_N_ELEMENTS(command_list))  {
+		const char *name = command_list[idx].name;
+		const char *alias = command_list[idx].alias;
+
+		idx++;
+		if (spa_strneq(name, text, len) || spa_strneq(alias, text, len))
+			return strdup(name);
+	}
+
+	return NULL;
+}
+
+static char **
+readline_command_completion(const char *text, int start, int end)
+{
+	char **matches = NULL;
+
+	/* Only try to complete the first word in a line */
+	if (start == 0)
+		matches = rl_completion_matches(text, readline_match_command);
+
+	/* Don't fall back to filename completion */
+	rl_attempted_completion_over = true;
+
+	return matches;
+}
+
+static void readline_init()
+{
+	rl_attempted_completion_function = readline_command_completion;
+	rl_callback_handler_install(">> ", readline_process_line);
+}
+
+static void readline_cleanup()
+{
+	rl_callback_handler_remove();
+}
+
+static void do_quit_on_signal(void *data, int signal_number)
 {
 	struct data *d = data;
 	d->quit = true;
@@ -2947,11 +3013,11 @@ static void do_quit(void *data, int signal_number)
 
 static void show_help(struct data *data, const char *name)
 {
-        fprintf(stdout, "%s [options] [command]\n"
+        fprintf(stdout, _("%s [options] [command]\n"
 		"  -h, --help                            Show this help\n"
 		"      --version                         Show version\n"
 		"  -d, --daemon                          Start as daemon (Default false)\n"
-		"  -r, --remote                          Remote daemon name\n\n",
+		"  -r, --remote                          Remote daemon name\n\n"),
 		name);
 
 	do_help(data, "help", "", NULL);
@@ -3007,8 +3073,8 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 	l = pw_main_loop_get_loop(data.loop);
-	pw_loop_add_signal(l, SIGINT, do_quit, &data);
-	pw_loop_add_signal(l, SIGTERM, do_quit, &data);
+	pw_loop_add_signal(l, SIGINT, do_quit_on_signal, &data);
+	pw_loop_add_signal(l, SIGTERM, do_quit_on_signal, &data);
 
 	spa_list_init(&data.remotes);
 	pw_map_init(&data.vars, 64, 16);
@@ -3018,6 +3084,10 @@ int main(int argc, char *argv[])
 				PW_KEY_CORE_DAEMON, daemon ? "true" : NULL,
 				NULL),
 			0);
+	if (data.context == NULL) {
+		fprintf(stderr, "Can't create context: %m\n");
+		return -1;
+	}
 
 	pw_context_load_module(data.context, "libpipewire-module-link-factory", NULL, NULL);
 
@@ -3029,12 +3099,16 @@ int main(int argc, char *argv[])
 	if (optind == argc) {
 		data.interactive = true;
 
-		pw_loop_add_io(l, STDIN_FILENO, SPA_IO_IN|SPA_IO_HUP, false, do_input, &data);
-
 		fprintf(stdout, "Welcome to PipeWire version %s. Type 'help' for usage.\n",
 				pw_get_library_version());
 
+		readline_init();
+
+		pw_loop_add_io(l, STDIN_FILENO, SPA_IO_IN|SPA_IO_HUP, false, do_input, &data);
+
 		pw_main_loop_run(data.loop);
+
+		readline_cleanup();
 	} else {
 		char buf[4096], *p, *error;
 
@@ -3046,7 +3120,7 @@ int main(int argc, char *argv[])
 
 		pw_main_loop_run(data.loop);
 
-		if (!parse(&data, buf, p - buf, &error)) {
+		if (!parse(&data, buf, &error)) {
 			fprintf(stdout, "Error: \"%s\"\n", error);
 			free(error);
 		}
